#!/bin/bash

VERSION="(NCID) XxXxX"

########################
# Function definitions #
########################

usage() {
   #printf -v spcs "%*s" ${#prgE} " "   # indent
   #indent example: $spcs  Options MUST appear before message.

   read -r -d '' _usage <<EOF

Usage: $prgE [options]

       Searches for tclsh/wish binaries installed with macOS Homebrew and
       creates proper tclsh/wish symbolic links in /usr/local/bin. This
       script only needs to be run once for a new NCID installation, or
       when new binary versions are installed by Homebrew.

       Homebrew does not automatically create any tclsh/wish symbolic
       links because doing so could conflict with the symbolic links
       already present when macOS is installed.

       Homebrew allows installing two variants of TCL/TK, one being compiled
       for the macOS native Aqua windowing system and the other being compiled
       for the X11 windowing system. (See install steps below.) This script 
       identifies both by examining the path that Homebrew used for the
       installation. At the time of this writing it is necessary to install
       both variants because the Aqua variant is bundled with the 'bwidget'
       TK graphical user interface package; the X11 variant does not.

       For each binary found, two symbolic links will be created in
       /usr/local/bin using the following naming convention:

           {tclsh|wish}

           {tclsh|wish}<full version number><windowing system>

       Examples:

           tclsh8.6.9x11    with symbolic link ==> tclsh
           wish8.6.9x11     with symbolic link ==> wish

           tclsh8.6.6aqua
           wish8.6.6aqua

       Only the presence of binaries compiled for the X11 windowing system
       will ultimately result in tclsh/wish (i.e., no version info in their
       names) symbolic links being created in /usr/local/bin. If multiple
       X11 versions are found, the one with the highest version number will
       be used.

       No attempt is made to make proper symbolic links to the manual (man)
       pages.

       Steps to install TCL/TK using Homebrew are below. You may want to
       check the NCID online documention for the latest updates:
       https://ncid.sourceforge.io/doc/NCID-UserManual.html#instl_mac_top

            brew tap sethrfore/homebrew-r-srf

            brew install sethrfore/r-srf/tcl-tk-x11 bwidget

            brew install --cask xquartz


Options:
       [-h]
       [-R] [-v] [-V] [-x]

       -h = show this help

       -R = remove all tclsh/wish symbolic links and exit

       -v = verbose output (probably of use only to developers)

       -V = show version and exit

       -x = dry run
 
EOF

   _term_lines=$(tput lines)
   _usage_lines=$(echo "$_usage" | wc -l)
   [ $_usage_lines -lt $_term_lines ] && echo "$_usage" ||  echo "$_usage" | less $less_options

   exit 1
}

progexists()
#https://stackoverflow.com/a/34143401/2329642
{
  command -v "$1" >/dev/null 2>&1
}

abort_on_error() {

   rc=$? ; [ $rc -ne 0 ] && echo $use_e "\nerror $rc" && exit 1

}
          
user_pause() {

   echo $use_e "Hit ENTER to continue:\c"
   read junk
   echo $use_e "\n\n\n"
   return 0

}

show_conflict_count_args() {

   # example: [ $output_to_console -eq 1 ] && show_conflict_count_args "These options are mutually exclusive:" conflict_count_gGioRst

   _text="$1"
   _var=$2

   echo $use_e "${_text}\c"

   _args=$(echo "$_var" | cut -d_ -f3)
   _args_len=${#_args}

   for (( _i=1; _i<=${_args_len}; _i++ )); do
     echo $use_e " -${_args:$_i-1:1}\c"
     [ $_i -ne $_args_len ] && echo $use_e ",\c"
   done
   echo

}

normalize_it() {
                      
   # pass numeric version string in $1
   # returns normalize value on stdout
       
   _text_in=$(echo "$1" | tr '_' '.')
 
   #https://stackoverflow.com/a/7816952/2329642
   IFS=. read piece1 piece2 piece3 piece4 <<< "$_text_in"
       
   [ -z "$piece3" ] && piece3="0" # if the passed version is x.y and not x.y.z then assume 0 for z
   [ -z "$piece4" ] && piece4="0" # if the passed version is x.y.z_n and not x.y.z then assume 0 for n
          
   printf "%02d%02d%02d%02d" ${piece1} ${piece2} ${piece3} ${piece4}

}

right_justify() {

   printf "%*s" $right_justify_pad "$*"

}

###############################
# End of function definitions #
###############################

prog=`basename $0 .sh` # without extension
prgE=`basename $0`     # with extension

# Test whether echo needs option "-e" to interpret new line "\n"
if [ "`echo -e`" != "-e" ] ; then use_e=" -e" ; else use_e= ; fi

equals="==============================================================================="

#check if 'less' has limited functionality
  less_options="-M -e -F"
  less -h 2>&1 | grep -q -e '-ENh~'
  rc=$?
  if [ $rc -eq 0 ]
     then
        less_options="-E"
  fi

###########################
# Options on command line #
###########################

  output_to_console=1

  version=$VERSION # because $0.sub resets VERSION

  conflict_count=0
  show_usage=0
  show_version=0
  dry_run=0
  do_remove_only=0
  verbose=0
 
  # use in 'case' below if needed:
  #          ((conflict_count++))
  #          ;;

  while getopts :hRvVx opt ; do
      case $opt in
          h) show_usage=1;;
          R) do_remove_only=1;;
          v) verbose=1;;
          V) show_version=1;;
          x) dry_run=1;;
         \?) echo "Invalid option: -$OPTARG"
             show_usage=1;;
          :) echo "Option -$OPTARG requires an argument."
             show_usage=1;;
      esac
  done

  [ $verbose -ne 0 ] && output_to_console=1

  [ $verbose -ne 0 ] && echo "Original args= [$#] $*"

  shift $((OPTIND-1)) # skip over command line args (if any)

  [ $verbose -ne 0 ] && echo "After getopts= [$#] $*"
 
  if [ $show_version -eq 1 ]
     then
        #https://stackoverflow.com/a/10415158/2329642
        ver_len1=${#prgE};ver_len2=$((${#prog}+4))
        ver_pad=$(echo $(($ver_len1>$ver_len2 ? $ver_len1 : $ver_len2)))
        printf "%-*s: VERSION=%s\n" $ver_pad $prgE "$VERSION"
        #source ${script_dir}/${prog}.sub
        exit 1
  fi

  abort=0

  if [ $conflict_count -gt 1 ]
     then
        [ $output_to_console -eq 1 ] && show_conflict_count_args "These options are mutually exclusive:" conflict_count
        abort=1
  fi
 
  # the following variables need to be initialized here so they can be shown in usage


  [ $show_usage -eq 1 ] && usage

  [ $abort -ne 0 ] && exit $abort # may want to show usage instead of aborting

  [ $dry_run -eq 1 ] && [ $output_to_console -eq 1 ] && echo $use_e "\n***** DRY RUN *****\n"

  # add more checks here that might necessitate showing usage
 
  [ $show_usage -eq 1 ] && usage 

########################
##### MAIN ROUTINE #####
########################

right_justify_pad=50

# establish global variables

  highest_aqua_ver_string=
  highest_aqua_ver_normalized=0

  highest_x11_ver_string=
  highest_x11_ver_normalized=0

# example 'find' output:
#/usr/local/Cellar/tcl-tk/8.6.9/bin/wish8.6
#/usr/local/Cellar/tcl-tk/8.6.9/bin/wish
#/usr/local/Cellar/tcl-tk/8.6.9/bin/tclsh8.6
#/usr/local/Cellar/tcl-tk/8.6.9/bin/tclsh
#/usr/local/Cellar/tcl-tk/8.6.13/bin/wish8.6
#/usr/local/Cellar/tcl-tk/8.6.13/bin/wish
#/usr/local/Cellar/tcl-tk/8.6.13/bin/tclsh8.6
#/usr/local/Cellar/tcl-tk/8.6.13/bin/tclsh
#/usr/local/Cellar/tcl-tk-x11/8.6.8/bin/wish8.6
#/usr/local/Cellar/tcl-tk-x11/8.6.8/bin/wish
#/usr/local/Cellar/tcl-tk-x11/8.6.8/bin/tclsh8.6
#/usr/local/Cellar/tcl-tk-x11/8.6.8/bin/tclsh
#/usr/local/Cellar/tcl-tk-x11/8.6.13/bin/wish8.6
#/usr/local/Cellar/tcl-tk-x11/8.6.13/bin/wish
#/usr/local/Cellar/tcl-tk-x11/8.6.13/bin/tclsh8.6
#/usr/local/Cellar/tcl-tk-x11/8.6.13/bin/tclsh

echo "Searching for all Homebrew tclsh/wish binary files..."
echo

while read -r -u5 full_path; do

      ver_pkg=$(echo "$full_path" | cut -d/ -f5) # tcl-tk or tcl-tk-x11
      ver_string=$(echo "$full_path" | cut -d/ -f6)

      filename=$(echo "$full_path" | cut -d/ -f8) # tclsh, tclsh8.6, wish, wish8.6

      #https://stackoverflow.com/questions/48131243/remove-digits-from-end-of-string
      filename_no_digits=$(echo "$filename" | sed 's/[0-9]*\([0-9].*\)//g')

      #echo "ver_pkg=$ver_pkg"
      #echo "ver_string=$ver_string"
      #echo "filename=$filename"
      #continue

      # filenames with no trailing version are symlinks so we ignore then
      if [ "$filename" = "tclsh" ] || [ "$filename" = "wish" ]; then
         continue
      fi

      ver_normalized=$(normalize_it $ver_string)

      # determine if path is for aqua or x11

        if [ "$ver_pkg" = "tcl-tk" ]; then

           ver_label=aqua
           if [ $ver_normalized -gt $highest_aqua_ver_normalized ]; then
              highest_aqua_ver_string=$ver_string
              highest_aqua_ver_normalized=$ver_normalized
           fi

        else

           ver_label=x11
           if [ $ver_normalized -gt $highest_x11_ver_normalized ]; then
              highest_x11_ver_string=$ver_string
              highest_x11_ver_normalized=$ver_normalized
           fi

        fi

      # make meaningful symbolic links

        actual="$full_path"
        #next line replaces old filename with version e.g., tclsh8.6 with full version string from path e.g. tclsh8.6.13
        sym="/usr/local/bin/${filename_no_digits}${ver_string}${ver_label}"

        if [ $dry_run -eq 0 ]; then

           if [ -e "$sym" ]; then 
              echo
              if [ -L "$sym" ]; then 
                 echo "Removing old symbolic link: $(right_justify "$sym") ==> $(readlink -n "$sym")"
                 rm "$sym"
                 abort_on_error
              else
                 echo "Leaving non-symbolic link: $(right_justify "$sym")"
                 continue
              fi
           fi

           if [ $do_remove_only -eq 0 ]; then

              echo
              echo "Creating new symbolic link: $(right_justify "$actual") ==> $sym"
              ln -s "$actual" "$sym"
              abort_on_error

           fi

        else

           if [ -e "$sym" ]; then 
              echo
              if [ -L "$sym" ]; then 
                 echo "WOULD HAVE removed old symbolic link: $(right_justify "$sym") ==> $(readlink -n "$sym")"
              else
                 echo "WOULD HAVE left non-symbolic link: $(right_justify "$sym")"
                 continue
              fi
           fi

           if [ $do_remove_only -eq 0 ]; then
              echo
              echo "WOULD HAVE created new symbolic link: $(right_justify "$actual") ==> $sym"
           fi

        fi    

done 5< <( find /usr/local/Cellar/tcl-tk /usr/local/Cellar/tcl-tk-x11 -name "tclsh*" -o -name "wish*" | fgrep /bin/ | sort )

if [ -z "${highest_x11_ver_string}" ]; then

   echo
   echo "No Homebrew installation of formula 'tcl-tk-x11' was found."
   echo
   echo "You probably need to install it. For instructions run:  $prgE -h"
   echo
   exit 1

fi

# finish by creating a single sym link for tclsh and a single sym link for wish, for the highest x11 version

  echo

  if [ $do_remove_only -eq 0 ]; then
     echo
     echo "Creating final symbolic links to latest X11 tclsh/wish files..."
     echo
  fi

  new_ver_string="${highest_x11_ver_string}x11"

  cd /usr/local/bin

  for i in tclsh wish; do

      if [ $dry_run -eq 0 ]; then

         if [ -e "$i" ]; then 
            echo
            if [ -L "$i" ]; then
               echo "Removing old symbolic link: $(right_justify "$(pwd)/${i}") ==> $(readlink -n "$i")"
               rm "$i"
               abort_on_error
            else
               echo "Leaving non-symbolic link: $(right_justify "$(pwd)/${i}")"
               continue
            fi
         fi

         if [ $do_remove_only -eq 0 ]; then

            echo
            echo "Creating new symbolic link: $(right_justify "$(pwd)/${i}") ==> ${i}${new_ver_string}"
            ln -s "${i}${new_ver_string}" "${i}"
            abort_on_error

         fi

      else

         if [ -e "$i" ]; then 
            echo
            if [ -L "$i" ]; then 
               echo "WOULD HAVE removed old symbolic link: $(right_justify "$(pwd)/${i}") ==> $(readlink -n "$i")"
            else
               echo "WOULD HAVE left non-symbolic link: $(right_justify "$(pwd)/${i}")"
               continue
            fi
         fi

         if [ $do_remove_only -eq 0 ]; then
            echo
            echo "WOULD HAVE created new symbolic link: $(right_justify "$(pwd)/${i}") ==> ${i}${new_ver_string}"
         fi

     fi

  done

echo
echo "Done."
