#!/bin/bash

VERSION="(NCID) XxXxX"

########################
# Function definitions #
########################

usage() {
   #printf -v spcs "%*s" ${#prgE} " "   # indent
   #indent example: $spcs  Options MUST appear before message.

# [-q]
#
#       -q = quiet (no output to console)
#
# [-v]
#       -v = verbose output (overrides -q)
#
#       -V = show version and exit (ignores -q)
#

# [-v 0|1]
#              0 = simple, no arg '-v'
#              1 = multi-level    '-v <0-9>'
#
#       Options MUST appear before ...

   read -r -d '' _usage <<EOF

Usage: $prgE [options]

       If no options are specified, the following changes are made:

          - if not already present, adds Terminal    to the XQuartz applications menu

          - if not already present, adds NCID Client to the XQuartz applications menu

          - makes NCID Client the app to run when XQuartz is launched

       Use the dry-run option -x to prevent changes.

Options:
       [-h]
       [-r <xterm|ncid|nothing>|-R] [-s] [-V] [-x]

       -h = show this help

       -r = sets which app to run when XQuartz is launched;
            default is '$app_to_run'

       -R = removes the XQuartz preference file $pref_file_for_help, then exits;
            you will need to launch XQuartz to recreate it with default
            settings

       -s = show the contents of XQuartz preference file $pref_file_for_help, then exits;
            uses the 'plutil' utility to convert it from binary format to text

       -V = show version and exit

       -x = dry run
 
EOF

   _term_lines=$(tput lines)
   _usage_lines=$(echo "$_usage" | wc -l)
   [ $_usage_lines -lt $_term_lines ] && echo "$_usage" ||  echo "$_usage" | less $less_options

   exit 1
}

progexists()
#https://stackoverflow.com/a/34143401/2329642
{
  command -v "$1" >/dev/null 2>&1
}

abort_on_error() {

   rc=$? ; [ $rc -ne 0 ] && echo $use_e "\nerror $rc" && exit 1

}
          
not_imp() {

   echo "WARNING: '$1' is not currently implemented and will be ignored."

}

user_pause() {

   echo $use_e "Hit ENTER to continue:\c"
   read junk
   echo $use_e "\n\n\n"
   return 0

}

show_conflict_count_args() {

   # example: [ $output_to_console -eq 1 ] && show_conflict_count_args "These options are mutually exclusive:" conflict_count_gGioRst

   _text="$1"
   _var=$2

   echo $use_e "${_text}\c"

   _args=$(echo "$_var" | cut -d_ -f3)
   _args_len=${#_args}

   for (( _i=1; _i<=${_args_len}; _i++ )); do
     echo $use_e " -${_args:$_i-1:1}\c"
     [ $_i -ne $_args_len ] && echo $use_e ",\c"
   done
   echo

}

match_foundv5() {

  # usage: match_foundv5 <string1|-f filename> <string2|-e string2> [-i]
  #
  # look for <string2> in either <string1> or <filename>
  # pass -i for case insensitive search
  # return code=0 (true) if match was successful
  # otherwise  =1 (false)
  #
  # if you specify -e then <string2> can include a grep expression

  _cli_e=
  _cli_f=0
  _cli_i=
   
  if [ "$1" = "-f" ]; then
        _cli_f=1
        shift
  fi

  _string1="$1"
  shift

  _string2="$1"
  shift

  if [ "$_string2" = "-e" ]; then
     _cli_e=$_string2
     _string2="$1"
     shift
  fi

  _cli_i="$1"

  if [ $_cli_f -eq 1 ]; then
     grep -q $_cli_i $_cli_e "$_string2" "$_string1"
     _rc=$?
  else
     echo "$_string1" | grep -q  $_cli_i $_cli_e "$_string2"
     _rc=$?
  fi

  if [ $_rc -eq 0 ] # <-- this is the main difference between match_foundv5 and match_not_foundv5
     then
        #[ $verbose -ne 0 ] && echo "Found '$_string2' in '$_string1'"
        return 0
     else
        #[ $verbose -ne 0 ] && echo "Did not find '$_string2' in '$_string1'"
        return 1
  fi

}

match_not_foundv5() {

  # same usage as match_foundv5
  #
  # return code=0 (true) if match was NOT successful
  # otherwise  =1 (false)
  #

  _cli_e=
  _cli_f=0
  _cli_i=
   
  if [ "$1" = "-f" ]; then
        _cli_f=1
        shift
  fi

  _string1="$1"
  shift

  _string2="$1"
  shift

  if [ "$_string2" = "-e" ]; then
     _cli_e=$_string2
     _string2="$1"
     shift
  fi

  _cli_i="$1"

  if [ $_cli_f -eq 1 ]; then
     grep -q $_cli_i $_cli_e "$_string2" "$_string1"
     _rc=$?
  else
     echo "$_string1" | grep -q  $_cli_i $_cli_e "$_string2"
     _rc=$?
  fi

  if [ $_rc -ne 0 ] # <-- this is the main difference between match_foundv5 and match_not_foundv5
     then
        #[ $verbose -ne 0 ] && echo "Did not find '$_string2' in '$_string1'"
        return 0
     else
        #[ $verbose -ne 0 ] && echo "Found '$_string2' in '$_string1'"
        return 1
  fi

}


###############################
# End of function definitions #
###############################

prog=`basename $0 .sh` # without extension
prgE=`basename $0`     # with extension

# Test whether echo needs option "-e" to interpret new line "\n"
if [ "`echo -e`" != "-e" ] ; then use_e=" -e" ; else use_e= ; fi

equals="==============================================================================="
stars="*******************************************************************************"

#check if 'less' has limited functionality
  less_options="-M -e -F"
  less -h 2>&1 | grep -q -e '-ENh~'
  rc=$?
  if [ $rc -eq 0 ]
     then
        less_options="-E"
  fi

###########################
# Options on command line #
###########################

  output_to_console=1

  [ -t 0 ]; rc=$?; stdin_redirected=$rc
  [ -t 1 ]; rc=$?; stdout_redirected=$rc
  [ -t 2 ]; rc=$?; stderr_redirected=$rc

  version=$VERSION # because $0.sub resets VERSION

  conflict_count_rRs=0
  show_usage=0
  show_version=0
  dry_run=0
  verbose=0
  app_to_run=ncid
  app_to_run_binary=
  remove_pref_file=0
  show_pref_file=0
 
  # use in 'case' below if needed:
  #          ((conflict_count++))
  #          ;;

  while getopts :hqr:RsvVx opt ; do
      case $opt in
          h) show_usage=1;;
          q) not_imp -$opt
             #output_to_console=0
             ;;
          r) app_to_run=$OPTARG
             ((conflict_count_rRs++))
             ;;
          R) remove_pref_file=1
             ((conflict_count_rRs++))
             ;;
          s) show_pref_file=1
             ((conflict_count_rRs++))
             ;;
          v) not_imp -$opt
             #verbose=1
             ;;
          V) show_version=1;;
          x) #not_imp -$opt
             dry_run=1
             ;;
         \?) echo "Invalid option: -$OPTARG"
             show_usage=1;;
          :) echo "Option -$OPTARG requires an argument."
             show_usage=1;;
      esac
  done

  [ $verbose -ne 0 ] && output_to_console=1

  [ $verbose -ne 0 ] && echo "Original args= [$#] $*"

  shift $((OPTIND-1)) # skip over command line args (if any)

  [ $verbose -ne 0 ] && echo "After getopts= [$#] $*"
 
  if [ $show_version -eq 1 ]
     then
        #https://stackoverflow.com/a/10415158/2329642
        ver_len1=${#prgE};ver_len2=$((${#prog}+4))
        ver_pad=$(echo $(($ver_len1>$ver_len2 ? $ver_len1 : $ver_len2)))
        printf "%-*s: VERSION=%s\n" $ver_pad $prgE "$VERSION"
        #source ${script_dir}/${prog}.sub
        exit 1
  fi

  abort=0

  if [ $conflict_count_rRs -gt 1 ]
     then
        [ $output_to_console -eq 1 ] && show_conflict_count_args "These options are mutually exclusive:" conflict_count_rRs
        abort=1
  fi
 
  # the following variables need to be initialized here so they can be shown in usage

    pref_domain=
    pref_dir=~/Library/Preferences
    pref_file=
    pref_file_for_help="*** NOT FOUND ***"

    #https://www.xquartz.org/FAQs.html#suppressing-xterm-launching-by-default  
    for i in org.x.X11_launcher org.macosforge.xquartz.X11 org.x.X11 org.macports.X11 org.xquartz.X11; do
        x="${pref_dir}/${i}.plist"
        if [ -f "$x" ]; then
           pref_domain=$i
           pref_file="$x"
           pref_file_for_help="${i}.plist"
           break
        fi
    done

    xterm_app_to_run_binary=/opt/X11/bin/xterm
    ncid_app_to_run_binary=/usr/local/bin/ncid

    if [ -n "$app_to_run" ]; then
       case $app_to_run in
            xterm) app_to_run_binary=$xterm_app_to_run_binary;;
             ncid) app_to_run_binary=$ncid_app_to_run_binary;;
          nothing) app_to_run_binary=/usr/bin/true;;
                *) echo "Invalid app name for option -r: $app_to_run"
                   echo "You must specify one of: xterm, ncid or the literal word \"nothing\" (without quotes)"
                   abort=1
                   ;;      
       esac
    fi

  [ $show_usage -eq 1 ] && usage

  [ $abort -ne 0 ] && exit $abort # may want to show usage instead of aborting

  [ $dry_run -eq 1 ] && [ $output_to_console -eq 1 ] && echo $use_e "\n***** DRY RUN *****\n"

  # add more checks here that might necessitate showing usage
 
  [ $show_usage -eq 1 ] && usage 

########################
##### MAIN ROUTINE #####
########################

tmp1=/tmp/${prog}-1.$$

ps ax > $tmp1

if match_foundv5 -f $tmp1 "bin/Xquartz"; then
   echo
   echo "$stars"
   echo "*   WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING   *"
   echo "$stars"
   echo
   echo "XQuartz appears to be currently running. You should"
   echo "only run this script when it is not running."
   if [ $dry_run -eq 0 ]; then
      echo
      echo "Switching to DRY RUN mode in order to continue."
      echo
      dry_run=1
   fi
fi

if [ $dry_run -ne 0 ]; then
   dry_run_tag="WOULD HAVE set "
   dry_run_indent="$(echo "$dry_run_tag" | tr '[:alpha:]' ' ')"
fi

abort=0
if [ -z "$pref_file" ]; then
   echo "Could not find a preference file for XQuartz."
   abort=1
else
   if [ ! -f "$pref_file" ]; then
      echo "File not found: $pref_file"
      abort=1
   fi
fi

[ $abort -ne 0 ] && echo "Maybe you need to launch XQuartz to create the default preference file?"

for i in $xterm_app_to_run_binary $ncid_app_to_run_binary $app_to_run_binary; do
    if [ ! -f "$i" ]; then
       echo "File not found: $i"
       abort=1
    fi
done

[ $abort -ne 0 ] && exit $abort

if [ $remove_pref_file -eq 1 ]; then
   if [ $dry_run -eq 0 ]; then
      rm "$pref_file"
      abort_on_error
      echo "Preference file has been removed: $pref_file"
      echo "You need to launch XQuartz to create the default preference file."
   else
      echo "WOULD HAVE removed preference file $pref_file"
   fi
   exit 1
fi

if [ $show_pref_file -eq 1 ]; then
   echo
   echo "Contents of $pref_file:"
   echo
   plutil -p $pref_file | awk '{print "     " $0}'
   echo
   exit 1
fi

old_apps_menu="$(defaults read $pref_file apps_menu 2>&1)"    ; rc=$? ; [ $rc -ne 0 ] &&  old_apps_menu="*** NOT SET ***"
old_app_to_run="$(defaults read $pref_file app_to_run 2>&1 )" ; rc=$? ; [ $rc -ne 0 ] && old_app_to_run="*** NOT SET ***"

new_menu_xterm_app_name="Terminal"
new_menu_xterm_binary="xterm"
new_menu_xterm_shortcut="n"

new_menu_ncid_app_name="Network Caller ID Client"
new_menu_ncid_binary="$ncid_app_to_run_binary"
new_menu_ncid_shortcut=""

new_app_to_run_binary="$app_to_run_binary"

echo

xterm_already_in_menu=0
if match_foundv5 "$old_apps_menu" "$new_menu_xterm_binary"; then
   echo "XQuartz application menu already has: $new_menu_xterm_app_name"
   xterm_already_in_menu=1
else
   echo "XQuartz application menu does not have: $new_menu_xterm_app_name"
fi

menu_xterm_data="-array-add '( \"$new_menu_xterm_app_name\", $new_menu_xterm_binary, \"$new_menu_xterm_shortcut\" )'"

ncid_already_in_menu=0
if match_foundv5 "$old_apps_menu" "$new_menu_ncid_binary"; then
   echo "XQuartz application menu already has: $new_menu_ncid_app_name"
   ncid_already_in_menu=1
else
   echo "XQuartz application menu does not have: $new_menu_ncid_app_name"
fi

menu_ncid_data="-array-add '( \"$new_menu_ncid_app_name\", $new_menu_ncid_binary, \"$new_menu_ncid_shortcut\" )'"

echo
echo "Old app to run:  $old_app_to_run"

changes_made=0

if [ $dry_run -eq 0 ]; then
   if [ $xterm_already_in_menu -eq 0 ]; then
      echo
      echo "New menu choice: $new_menu_xterm_app_name"
      echo "                 $new_menu_xterm_binary"
      #echo "                 $new_menu_xterm_shortcut"
      eval defaults write $pref_file apps_menu  $menu_xterm_data
      changes_made=1
   fi
   if [ $ncid_already_in_menu -eq 0 ]; then
      echo
      echo "New menu choice: $new_menu_ncid_app_name"
      echo "                 $new_menu_ncid_binary"
      #echo "                 $new_menu_ncid_shortcut"
      eval defaults write $pref_file apps_menu  $menu_ncid_data
      changes_made=1
   fi
   if [ -n "$app_to_run_binary" ]; then
      echo
      echo "New app to run:  $new_app_to_run_binary"
      defaults write $pref_file app_to_run $new_app_to_run_binary
      changes_made=1
   fi
else
   if [ $xterm_already_in_menu -eq 0 ]; then
      echo
      echo "${dry_run_tag}New menu choice: $new_menu_xterm_app_name"
      echo "${dry_run_indent}                 $new_menu_xterm_binary"
      #echo "${dry_run_indent}                 $new_menu_xterm_shortcut"
   fi
   if [ $ncid_already_in_menu -eq 0 ]; then
      echo
      echo "${dry_run_tag}New menu choice: $new_menu_ncid_app_name"
      echo "${dry_run_indent}                 $new_menu_ncid_binary"
      #echo "${dry_run_indent}                 $new_menu_ncid_shortcut"
   fi
   if [ -n "$app_to_run_binary" ]; then
      echo
      echo "${dry_run_tag}New app to run:  $new_app_to_run_binary"
   fi
fi

echo

if [ $changes_made -eq 0 ]; then
   echo "No changes have been made."
else
   echo "Preference file has been updated: $pref_file"
fi
