#!/usr/bin/perl

# cidupdate - update Caller ID call log file or files using the
# current alias file.
#
# Created by Aron Green on Mon Nov 25, 2002
#
# Copyright (c) 2002-2018 by
#   Aron Green,
#   John L. Chmielewski <jlc@users.sourceforge.net> and
#   Steve Limkemann
#   Chris Lenderman

BEGIN {
    # causes perl runtime errors to be included in STDOUT
    close (STDERR);
    open(STDERR, ">&STDOUT");
}

use strict;
use warnings;
use Pod::Usage;
use File::Basename;
use Getopt::Long qw(:config no_ignore_case_always);

my (@aliases, $alias, $cidlog, $newcidlog, $changed);
my ($help, $man, $version, $multiple, $ignore1, $regex);
my ($type, $from, $to, $depend, $mod_time, $logType);
my ($date, $time, $line, $number, $mesg, $name, @log_files);
my ($htype, $scall, $ecall, $ctype, $mtype, $message, $info);

my $prog = basename($0);
my $VERSION = "(NCID) XxXxX";

my $ALIAS = "/etc/ncid/ncidd.alias";
my $CIDLOG = "/var/log/cidcall.log";

my $debug = 1;
my $verbose = 1;
my $lineCount = 0;
my $lineCountModulo = 100;


select(STDERR); $| = 1; # enable autoflush, otherwise output to STDERR
select(STDOUT); $| = 1; # may appear before output to STDOUT

Getopt::Long::Configure ("bundling");
my ($result) = GetOptions(
    'help|h'        => \$help,
    'man|m'         => \$man,
    'debug|D'       => \$debug,
    'verbose|v=i'   => \$verbose,
    'version|V'     => \$version,
    'aliasfile|a=s' => \$alias,
    'cidlog|c=s'    => \$cidlog,
    'ignore1|r'     => \$ignore1,
    'multi'         => \$multiple,
    'regex|R=i'     => \$regex
 ) || pod2usage(2);
die "$prog $VERSION\n" if $version;
pod2usage(-verbose => 1, -exitval => 0) if $help;
pod2usage(-verbose => 2, -exitval => 0) if $man;

$alias = $ALIAS if !defined $alias;
$cidlog = $CIDLOG if !defined $cidlog;

@log_files = glob "$cidlog*";
die "Terminated: missing file: $cidlog\n" if $#log_files == -1;

$#log_files = 0 unless $multiple;

$SIG{'HUP'}  = 'sigHandle';
$SIG{'INT'}  = 'sigHandle';
$SIG{'QUIT'} = 'sigHandle';
$SIG{'TERM'} = 'sigHandle';
$SIG{'PIPE'} = 'sigHandle';

logMsg(3,"Processing alias file '$alias'\n\n");
$lineCount = 0;

open(ALIASFILE, $alias) || errorExit("Could not open $alias: $!\n");

while (<ALIASFILE>) {
    $lineCount++;
    if ($lineCount % $lineCountModulo == 0) { logMsg(7,"\n==> '$alias' continues <==\n\n"); }
    logMsg(5,sprintf("[%6d] %s", $lineCount, $_));
    next unless /^alias/;
    chomp;
    s/\s+#.+$//;
    if (/^alias\s+(\w+)\s+"?([^"]+)"?\s+=\s+"?([^"]+)"?\s+if\s+"?([^"]+)"?$/) {
        # alias NAME|NMBR "from" = "to" if "depend"
        ($type, $from, $to, $depend) = ($1, $2, $3, $4);
    } elsif (/^alias\s+(\w+)\s+"?([^"]+)"?\s+=\s+"?([^"]+)"?$/) {
        # alias NAME|NMBR "from" = "to"
        ($type, $from, $to, $depend) = ($1, $2, $3, '');
    } elsif (/^alias\s+"?([^"]+)"?\s+=\s+"?([^"]+)"?$/) {
        # alias "from" = "to"
        ($type, $from, $to, $depend) = ('NMBRNAME', $1, $2, '');
    } else {
        logMsg(1,"Unknown: $_\n");
        next;
    }
    $to =~ s/\s*$//;
    $depend =~ s/\s*$//;
    push @aliases, ([$type, $from, $to, $depend]);
}

#CID: *DATE*11242002*TIME*2112*LINE*1*NMBR*9549142285*MESG*NONE*NAME*Cell*

foreach $cidlog (@log_files) {
    unlink $cidlog if $cidlog =~ /\.new\.new$/;
    next if $cidlog =~ /\.new$/;
    $newcidlog = "$cidlog.new";
    open(CIDLOG, $cidlog) || errorExit("Could not open $cidlog: $!\n");
    open(NEWCIDLOG, ">$newcidlog") || errorExit("Could not open $newcidlog: $!\n");

    logMsg(3,"\n\nProcessing log file '$cidlog'\n\n");
    $lineCount = 0;

    while (<CIDLOG>) {
        $lineCount++;
        if ($lineCount % $lineCountModulo == 0) { logMsg(7,"\n==> '$cidlog' continues <==\n\n"); }
        logMsg(5,sprintf("[%6d] %s", $lineCount, $_));
        if (! /DATE/) {
            print NEWCIDLOG;
            next;
        }
        if (/^BLK:|^CID:|^END:|^HUP:|^MWI:|^OUT:|^PID:|^PUT:|^RID:|^WID:|^MSG:|^NOT:|^RLY:/) {
            ($logType, $date, $time, $line, $number, $mesg, $name) =
                (split /\*/) [0, 2, 4, 6, 8, 10, 12];
            if (/^END/) {
                ($logType, $htype, $date, $time, $scall, $ecall, $ctype, $line, $number, $name) =
                (split /\*/) [0, 2, 4, 6, 8, 10, 12, 14, 16, 18];
            }
            if (/^MSG|^NOT/) {
                # https://stackoverflow.com/questions/14127282/match-from-last-occurrence-using-regex-in-perl
                # allow 1 or more *'s in $message
                ($message, $info) = /(.*)(\*\*\*.*?$)/;

                # get the data fields from $info
                ($date, $time, $name, $number, $line, $mtype) = 
                (split /\*/, $info) [4, 6, 8, 10, 12, 14];

                ($logType) = $message =~ /(^\w+:\s)/;
                $message =~ s/^\w+:\s//;
            }

            if ($number eq 'RING') {
                print NEWCIDLOG;
                next;
            }
            foreach $alias (@aliases) {
                ($type, $from, $to, $depend) = @$alias;
                if ($type eq "NAME" && $depend) {
                    if (strmatch($number, $depend) && !strmatch($name, $to)) {
                        record_change ("(NAMEDEP) Changed \"$name\" to \"$to\" for $number", $cidlog);
                        $name = $to;
                    }
                }
                elsif ($type eq "NAME") {
                    if (strmatch($name, $from) && !strmatch($name, $to)) {
                        record_change ("(NAMEONLY) Changed \"$name\" to \"$to\"", $cidlog);
                        $name = $to;
                    }
                }
                if ($type eq "NMBR" && $depend) {
                    if (strmatch($name, $depend) && !strmatch($number, $to)) {
                        record_change ("(NMBRDEP) Changed \"$number\" to \"$to\" for $name", $cidlog);
                        $number = $to;
                    }
                }
                elsif ($type eq "NMBR") {
                    if (strmatch($number, $from) && !strmatch($number, $to)) {
                        record_change ("(NMBRONLY) Changed \"$number\" to \"$to\"", $cidlog);
                        $number = $to;
                    }
                }
                if ($type eq "LINE") {
                    if (strmatch($line, $from) && !strmatch($line, $to)) {
                        record_change ("(LINEONLY) Changed \"$line\" to \"$to\"", $cidlog);
                        $line = $to;
                    }
                }
                if ($type eq "NMBRNAME") {
                    if (strmatch($name, $from) && !strmatch($name, $to)) {
                        record_change ("(NMBRNAME:NAME) Changed \"$name\" to \"$to\"", $cidlog);
                        $name = $to;
                    }
                    if (strmatch($number, $from) && !strmatch($number, $to)) {
                        record_change ("(NMBRNAME:NMBR) Changed \"$number\" to \"$to\"", $cidlog);
                        $number = $to;
                    }
                }
            }
            if ($logType eq "END: ") {
                print NEWCIDLOG
                  "$logType*HTYPE*$htype*DATE*$date*TIME*$time*SCALL*$scall*ECALL*$ecall*CTYPE*$ctype*LINE*$line*NMBR*$number*NAME*$name*\n";
            } elsif ($logType eq "MSG: " || $logType eq "NOT: "){
                print NEWCIDLOG
                  "$logType$message***DATE*$date*TIME*$time*NAME*$name*NMBR*$number*LINE*$line*MTYPE*$mtype*\n";
            } else {
                print NEWCIDLOG
                  "$logType*DATE*$date*TIME*$time*LINE*$line*NMBR*$number*MESG*$mesg*NAME*$name*\n";
            }
        } else {
            print NEWCIDLOG;
        }
    }
    no_change ($cidlog);
    $mod_time = (stat CIDLOG)[9];
    close CIDLOG;
    close NEWCIDLOG;
    utime $mod_time, $mod_time, $newcidlog;
}
report_changes ();
if ($changed) {
    if ( -t STDIN) {
        logMsg(1,"\nreject or accept changes? (R/a): ");
        my $resp = <STDIN>;
        chomp $resp;
        $resp = 'r' unless $resp;
        if (substr ((lc $resp), 0, 1) eq 'r') {
            remove_new ();
            logMsg(1, "\nUpdates to CID call logs have been discarded\n\n");
        } else {
            use_new ();
            logMsg(1, "\nChanges have been made to the CID log files\n\n");
        }
    }
} else {remove_new ();}

{
    my (%files, @files, %changes, @changes);

    sub record_change {
        my ($key, $file) = @_;

        push @files, $file unless exists $files{$file};
        $files{$file} ++;
        push @changes, $key unless exists $changes{$key};
        $changes{$key}++;
    }

    sub report_changes {
        my ($temp1, $temp2);

        logMsg(1, "\n") if -t STDIN;
        foreach (@files) {
            ($temp1, $temp2) = $files{$_} == 1 ? ('was', ''): ('were', 's');
            if ($files{$_} == 0) {$files{$_} = 'no';}
            else {$changed = 1;}
            logMsg(1, "There $temp1 $files{$_} change$temp2 to $_\n");
        }
        logMsg(1, "\n");
        foreach (@changes) {
            $temp1 = $changes{$_} == 1 ? '': 's';
            logMsg(1, "$_ $changes{$_} time$temp1\n");
        }
    }

    sub no_change {
        my $file = shift;

        if (not exists $files{$file}) {
            push @files, $file;
            $files{$file} = 0;
        }
    }

    sub remove_new {

        foreach (@files) {
            unlink "$_.new"
        }
    }

    sub use_new {

        foreach (@files) {
            if (/\.log.+$/) {
                # not the current call log
                rename "$_.new", $_;
            } elsif (system ("killall -SIGUSR1 ncidd") != 0) {
                # update the current call log if signal to ncidd failed
                rename "$_.new", $_;
            }
        }
    }
}

# This function is also used in ncidutil.pl.  If you modify
# it here, make the same modifications in ncidutil.pl.
sub strmatch {
    my($string, $find) = @_;

    # remove comment at end of the line
    $string =~ s/\s+#.*$//;

    # remove '?' at beginning of line, some phone systems generate ?<name>
    $string =~ s/^\?//; 
    $find =~ s/^\?//;

    if (defined $ignore1) {
        if ($find =~ /^1\?/) { $find =~ s/^1\?//; }
        else { $find =~ s/^1//; }
        $string =~ s/^1//;
    }
    if (!defined $regex) {
        # Simple Expressions
        if ($find =~ /^\^/) {
            # handle  ^<string> ^1?<string> ^*<string> ^*<string>* ^<string>*
            $find =~ s/\^\*/^.*/;
            if ($find =~ /\*$/) {$find =~ s/\*$//;}
            else {$find =~ s/$/\$/;}
        }else {
            # handle <string> 1?<string> *<string> *<string>* <string>*
            if ($find =~ /^\*/) {$find =~ s/\*//;}
            else {$find =~ s/^/\^/;}
            if ($find =~ /\*$/) {$find =~ s/\*$//;}
            else {$find =~ s/$/\$/;}
        }
        # escape regex characters not used
        $find =~ s/([+.()|{}\[\]-])/\\$1/g;
    }

    return ($string =~ /$find/);
}

sub logMsg {
    my($level, $message) = @_;

    if (!defined $message) {print "Oops, unexpected exit\n"; exit 1}

    # write to STDOUT
    print $message if $debug && $verbose >= $level;

    # write to logfile
    #print LOGFILE $message if $fileopen && $verbose >= $level;
}


sub cleanup {
    close CIDLOG;
    close NEWCIDLOG;
    remove_new ();
}

sub errorExit {
    cleanup ();
    die "Terminated: @_";
}

sub sigHandle {
    my $sig = shift;
    cleanup ();
    die "Terminated: Caught SIG$sig\n";
}

=head1 NAME

cidupdate -  update aliases in the NCID call file

=head1 SYNOPSIS

 cidupdate [--help|-h] [--man|-m] [--debug|-D]
           [--verbose|-v <1-9>] [--version|-V]
           
 cidupdate [--aliasfile | -a <file>]
           [--cidlog    | -c <file>]
           [--ignore1   | -i]
           [--regex     | -R]
           [--multi]

=head1 DESCRIPTION

The cidupdate script updates the current call log file
(cidcall.log) using the entries found in the alias file
(ncidd.alias).

If the "--multi" option is present, the current cidcall.log file
and previous call files are updated.

=head2 Options

=over 7

=item -h, --help

Displays the help message and exits.

=item -m, --man

Displays the manual page and exits.

=item -D, --debug

Debug mode. Always enabled, reserved for future use.

=item -v, --verbose <1-9>

Output information while processing is occurring.  Set
the level to a higher number for more information.  Levels range from
1 to 9, but not all levels are used.

Default: verbose = 1

=item -V, --version

Displays the version and exits.

=item -a <file>, --aliasfile <file>

Set the alias file to <file>

Default: /etc/ncid/ncidd.alias

=item -c <file>, --cidlog <file>

Set the call file to <file>

Default: /var/log/cidcall.log

=item -i, --ignore1

This is a US/Canada option only.

A leading one in an alias definition and in the calling number is ignored.

Normally an alias requires the calling number as it appears in the cidcall.log.
In the US a leading 1 may or may not be provided in incoming or outgoing calls.

Default: The leading 1 must be in the alias if it is in the calling number.

=item --multi

Updates the current cidcall.log file and all previous call files.

Default: Updates only the cidcall.log file

=item -R, --regex <0-2>

If regex = 0, uses Simple Expressions.

If regex = 1, uses Posix Regular Expressions.

If regex = 2, uses Perl-compatible Expressions.

Default: Uses Simple Expressions

=back

=head1 SEE ALSO

ncidd.conf.5,
ncidd.alias.5,
cidalias.1,
cidcall.1

=cut
