########################################################################
#                       PROCEDURE DEFINITIONS                          #
########################################################################


# http://www.androwish.org/index.html/tktview/cca4dc34ae71cd04c6d3904ab26d144723bfba7e
package require base64
proc read_binary_file {name} {
     set f [open $name rb]
     set result [read $f]
     close $f
     return $result
 }


proc normalizeTclVer {} {
    set tclVer ""
    foreach x [split [info patchlevel] .] {
       append tclVer [format "%02d" $x]
    }
    return $tclVer
}


proc checkFile {{whichFile} {warn ""}} {
    global abort
    if {![file exists $whichFile]} {
       logMsg 1 "Missing file:" $whichFile
       if {$warn == ""} {
         set abort 1
         tk_messageBox -message "Missing file: $whichFile" -icon warning -type ok
       }
    } else {
      logMsg 1 "Found file:" $whichFile
    }
}

proc checkScriptAndLoad {whichFile} {
    global abort
    if {![file exists $whichFile]} {
       logMsg 1 "Missing script:" $whichFile
       tk_messageBox -message "Missing script: $whichFile" -icon warning -type ok
       set abort 1
    } else {
      logMsg 1 "Loaded script:" $whichFile
      source $whichFile
    }
}

proc dumpFile {whichFile} {
   logMsg 1 "Contents of:"  $whichFile
   set inFile [open $whichFile r]
   while { [gets $inFile line] != -1 } {
       logMsg 1 "" "     $line"
   }
   close $inFile
}

# Handle verbosity levels
proc logMsg {level msg1 msg2} {
    global Verbose LogChan indent

    set msg2list [split $msg2 \n]
    foreach i $msg2list {
       set msg "[format "%-*s %s" $indent "$msg1" "$i"]"
       if {$Verbose >= $level} {
          puts $msg
          if {$LogChan != ""} {
              set systemTime [clock seconds]
              puts $LogChan "\[[clock format $systemTime -format "%m/%d %H:%M"]\] $msg"
          }
       }
       set msg1 ""
    }
}

proc doExit {} {
    global ncid_installer_is_running
    unset ncid_installer_is_running
    # check if being run from file manager and not launcher-ncid.tcl
    if {[pwd] == "/"} {exit}
}

########################################################################
#                      MAIN ROUTINE STARTS HERE                        #
########################################################################

if {[normalizeTclVer] <= "080607"} {
    tk_messageBox -message "[normalizeTclVer]\nYou are running TCL/TK version [info patchlevel]\nYou need to be running 8.6.8 or higher." -icon error -type ok
    doExit
}
set answer [tk_messageBox -message "About to install NCID -- proceed?"  -icon warning -type okcancel]
if { $answer != "ok" } {doExit}

global ncid_installer_is_running
set ncid_installer_is_running 1

set indent 18

set Verbose 1
set ScriptDir    [file normalize [file dirname [info script]]]
set ScriptBase   [file tail [file rootname [info script]]]
set ToolsDir     $ScriptDir
set MainDir      [file dirname $ToolsDir]
set ProtectedDir /data/data/tk.tcl.wish/files

set LogDir     [file join $MainDir "logs"]
if {![file isdirectory $LogDir]} {file mkdir $LogDir}
set LogFile [file join $LogDir "$ScriptBase.log"]
set LogChan [open $LogFile a+]

logMsg 1 "Starting:"      [file normalize [info script]]
logMsg 1 "Current dir:"   [pwd]
logMsg 1 "Script dir:"    $ScriptDir
logMsg 1 "Script base:"   $ScriptBase
logMsg 1 "Tools dir:"     $ToolsDir
logMsg 1 "Main dir:"      $MainDir
logMsg 1 "Protected dir:" $ProtectedDir
logMsg 1 "Log dir:"       $LogDir
logMsg 1 "Log file:"      $LogFile
logMsg 1 "TCL/TK ver:"    [info patchlevel]

set abort 0

set createWishrcScript [file join $ToolsDir create-protected-.wishrc.tcl]
set backupWishrcScript [file join $ToolsDir backup-protected-.wishrc.tcl]
set createWishrcTempl  [file join $ToolsDir protected-.wishrc.template]
set backupWishrcFile   [file join $ToolsDir protected-.wishrc.backup]
set getIpScript        [file join $ToolsDir getIp.tcl]
set getAndroidScript   [file join $ToolsDir getAndroidVer.tcl]
set protectedWishrc    [file normalize [file join $::env(HOME) .wishrc]]
set unprotectedWishrc  [file join $MainDir wishrc.tcl]
set iconPath           [file join $MainDir images ncid-192x192.png]
set execPath           [file join $MainDir ncid.tcl]
set plistScript        [file join $ToolsDir plist.tcl]

#possibly leftover from failed install
set badpref [file normalize [file join $ProtectedDir .ncid]]
if {[file exists $badpref]} {
    logMsg 1 "Deleting:"  $badpref
    file delete $badpref
}


checkScriptAndLoad $createWishrcScript
checkScriptAndLoad $backupWishrcScript
checkScriptAndLoad $getIpScript
checkScriptAndLoad $getAndroidScript
checkScriptAndLoad $plistScript
checkFile          $createWishrcTempl
checkFile          $backupWishrcFile warn
checkFile          $iconPath
checkFile          $execPath
checkFile          $protectedWishrc warn
checkFile          $unprotectedWishrc warn

if {$abort} {
   tk_messageBox -message "Installation aborted." -icon error -type ok
} else {
   logMsg 1 " " " "
   logMsg 1 "Executing:" $getIpScript
   logMsg 1 ""           [getIp 0]
   logMsg 1 " " " "
   logMsg 1 "Executing:" $getAndroidScript
   logMsg 1 ""           [getAndroidVer 0]

   logMsg 1 " " " "
   if {[file exists $backupWishrcFile]} {
   logMsg 1 "Backup file:" "Already exists! $backupWishrcFile"
   } else {
   logMsg 1 "Executing:" $backupWishrcScript
   logMsg 1 ""           [backupProtectedWishrc 0]
   }

   logMsg 1 " " " "
   logMsg 1 "Executing:" $createWishrcScript
   logMsg 1 ""           [createProtectedWishrc 0]
   logMsg 1 " " " "
   logMsg 1 "Executing:" $plistScript
   logMsg 1 "Display:"   [plist [borg displaymetrics] 2 ret]

   dumpFile $protectedWishrc

   if {[file exists $unprotectedWishrc]} {
     logMsg 1 " " " "
     dumpFile $unprotectedWishrc
   }

   set shortcutName "NCID Client"
   set icondata [read_binary_file $iconPath]
   set iconbase64 [::base64::encode -maxlen 0 $icondata]
   logMsg 1 " " " "
   logMsg 1 "Creating shortcut:"  '$shortcutName'
   logMsg 1 "         executes:"  '$execPath'
   logMsg 1 "         icon:"      '$iconPath'
   borg shortcut add "$shortcutName" file://$execPath $iconbase64

   logMsg 1 " " " "
   set tmp "NCID installed successfully."
   logMsg 1 "Terminated:" $tmp
   if {$LogChan != ""} {puts $LogChan "\n\n\n"}
   tk_messageBox -message $tmp -type ok
}

close $LogChan
doExit