/*
 * artech2ncid.h - This file is part of artech2ncid.
 *
 * Copyright (c) 2021,2023
 * by Bruno Grasland  <b.sletteland@orange.fr>
 *
 * Some parts of the code have been inspired from Ncid by
 * John L. Chmielewski <jlc@users.sourceforge.net>
 *
 * artech2ncid is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * artech2ncid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with artech2ncid.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <getopt.h>
#include <wchar.h>
#include <limits.h>
#include <netdb.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#ifdef __linux__
    #include <sys/prctl.h>
#endif
#include "version.h"
#include "libncid.h"

#if defined(__FreeBSD__)
    #include <netinet/in.h>
#endif

#include <hidapi/hidapi.h>

#if defined(__APPLE__)
    #include <hidapi/hidapi_darwin.h>
#endif

#ifndef HOST_NAME_MAX
    #define HOST_NAME_MAX 64 /* HOST_NAME_MAX is POSIX.1, Linux defines it as 64 */
#endif

#ifndef CONFIG
    #define CONFIG      "/etc/ncid/artech2ncid.conf"
#endif

#ifndef TRACEFILE
    #define TRACEFILE      "/var/log/artech2ncid.trace"
#endif


#define SHOWVER     "%s %s\n%s\n"
#define DESC        "%s - is an ncid gateway for the Artech AD102 device'\n"
#define NOOPT       "%s: not an option: %s\n"
#define USAGE       "\
Usage:   %s [options]\n\
Options: [-C <filename>      | --configfile <filename>]\n\
         [-d <0-0xF>         | --disable_incoming <0-0xF>]\n\
         [-D                 | --debug]\n\
         [-h                 | --help]\n\
         [-H <0|1>           | --hostname-flag <0|1>]\n\
         [-L <filename>      | --logfile <filename>]\n\
         [-m <0-2>           | --mem_trace <0-2>]\n\
         [-n <[host][:port]> | --ncid <[host][:port]>]\n\
         [-P <filename>      | --pidfile <filename>]\n\
         [-t                 | --test]\n\
         [-T <filename>      | --tracefile <filename>]\n\
         [-v <1-9>           | --verbose <1-9>]\n\
         [-V                 | --version]\n\
"

/* make sure BUFSIZ on non-Linux is the same as on Linux */
#ifndef _G_BUFSIZ
#undef  BUFSIZ
#define BUFSIZ      8192
#endif

#define NCIDPORT    3333
#define LOCALHOST   "127.0.0.1"
#define LINEID      "ARTECH"

#define FATAL       1
#define NOTFATAL    0

#define CIDSIZ      75

#define IDENT       "HELLO: IDENT: "
#define GWOUT       " [OUT]"
#define COMMAND     "HELLO: CMD: "



#ifndef PIDFILE
#define PIDFILE     "/var/run/artec2ncid.pid"
#endif

#ifndef LOGFILE
#define LOGFILE     "/var/log/artech2ncid.log"
#endif

#define NODESC      "No description available"


/* compiler option -Wformat-overflow gives warning for %s unless length
 * is used to reduce it from BUFSIZ (%s changed to %.300s)
 */
#define CALL        "CALL: ###DATE%.300s...CALL%.300s...LINE%.300s...NMBR%.300s...MESG%.300s...NAME%.300s+++\r\n"
#define CALLINFO    "CALLINFO: ###%.300s...DATE%.300s...SCALL%.300s...PCALL%.300s...ECALL%.300s...CALL%.300s...LINE%.300s...NMBR%.300s...NAME%.300s+++\r\n"

#define EQUALS      "==============================================================================="

#define MESSAGES    "Message-Waiting:"
#define MESSAGES_ON "Messages-Waiting: yes"
#define MESSAGES_OFF "Messages-Waiting: no"
#define QUOTE       '"'
#define NL          '\n'

#define NONAME      "NO NAME"
#define BADNAME     "BAD NAME"
#define MWINUMBER   "Voicemail"
#define MWINAME_ON  "Message(s) Waiting"
#define MWINAME_OFF "No Messages Waiting"
#define NONUMBER    "NO-NUMBER"
#define BADNUMBER   "BAD-NUMBER"
#define CALLOUT     "OUT"
#define CALLIN      "IN"
#define CANCEL      "CANCEL"
#define BYE         "BYE"

/*
 * Start of XDMF message definitions
 *
 * An XDMF message starts with one of the following bytes:
 *     SDMF_MESG_TYPE
 *     MDMF_MESG_TYPE
 *     MDMF_MSG_WAITING
 *
 * MDMF_MSG_WAITING is seldom used, it indicates voicemail
 * https://en.wikipedia.org/wiki/Message-waiting_indicator
 */

#define SDMF_MESG_TYPE    0x04
#define MDMF_MESG_TYPE    0x80
#define MDMF_MSG_WAITING  0x82

#define MDMF_VISUAL_IND   0x0B

#define MDMF_DATETIME     0x01
#define MDMF_CALLER_NMBR  0x02
#define MDMF_CALLER_NAME  0x07

#define MDMF_WHY_NO_NMBR  0x04
#define MDMF_WHY_NO_NAME  0x08

/*
 * Two reasons defined for (SDMF, MDMF) WHY_NO_NUMBER
 * and WHY_NO_NAME (MDMF)
 * WITHHELD    = 0x50   'P'RIVATE
 * UNAVAILABLE = 0x4F   'O'UT OF AREA
 */
#define LETTER_P          0x50
#define LETTER_O          0x4F

/*
 * CALL_TYPE_UK is only sent for ETSI FSK and BT FSK
 * In general it can be ignored.
 */
#define CALL_TYPE_UK      0x11
/*
 * There are three call subtypes defined:
 */
#define VOICE_UK          0x01
#define RINGBACK_UK       0x02
#define MSG_WAITING_UK    0x81

#define MDMF_NMSS_UK      0x13  /* Network Message System Status */

/* End of XDMF message definitions */

#define CIDDATE     0x01
#define CIDTIME     0x02
#define CIDNMBR     0x04
#define CIDNAME     0x08
#define CIDMESG     0x10
#define IGNHIGH     0x0F
#define CIDALL3     (CIDDATE | CIDTIME | CIDNMBR)
#define CIDALT3     (CIDDATE | CIDTIME | CIDNAME)
#define CIDALL4     (CIDDATE | CIDTIME | CIDNMBR | CIDNAME)
#define CIDNODT     (CIDNMBR | CIDNAME)


#define ONELINE    "POTS"

/* Call types received only from gateways (list 1 of 2) */
#define IN         30
#define CID        30
#define OUT        31
#define HUP        32
#define BLK        33
#define PID        34
#define PUT        35
#define WID        36
#define RID        37
#define MWI        38

/*report of incoming calls */
#define NO_CID 0x01
#define NO_WID 0x02
#define NO_RID 0x04
#define NO_MWI 0x08


/*
 * originally:
 * O - OUT-OF-AREA
 * A - ANONYMOUS
 * P - PRIVATE
 */
#define WORDFOR_O "Unavailable"
#define WORDFOR_A "ANONYMOUS"
#define WORDFOR_P "Withheld"


#define NONAME       "NO NAME"
#define NONMBR       "NO-NUMBER"
#define NOLINE       "NO-LINE"
#define NOTYPE       "USER"
#define NOMESG       "NONE"
#define NOCID        "No Caller ID"

#define CIDSIZE      272  //header( 1 byte ) + length (1 byte) + actual data (max 255 bytes) + checksum (1 byte)  + safety margin



