/*
 * cideasy2ncid.h - This file is part of cideasy2ncid.
 *
 * Copyright (c) 2024
 * by John L. Chmielewski <jlc@users.sourceforge.net>
 *
 * cideasy2ncid is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * cideasy2ncid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with cideasy2ncid.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <getopt.h>
#include <netdb.h>
#include <sys/fcntl.h>
#include <sys/socket.h>

#if defined(__FreeBSD__)
    #include <netinet/in.h>
#endif

#include <hidapi/hidapi.h>

#if defined(__APPLE__)
    #include <hidapi/hidapi_darwin.h>
#endif

#include "version.h"
#include "libncid.h"

#ifndef PIDFILE
    #define PIDFILE     "/var/run/cideasy2ncid.pid"
#endif

#ifndef LOGFILE
    #define LOGFILE     "/var/log/cideasy2ncid.log"
#endif

#ifndef CONFIG
    #define CONFIG      "/etc/ncid/cideasy2ncid.conf"
#endif

#ifndef HOST_NAME_MAX
    #define HOST_NAME_MAX 64 /* HOST_NAME_MAX is POSIX.1, Linux defines it as 64 */
#endif

/* make sure BUFSIZ on non-Linux is the same as on Linux */
#ifndef _G_BUFSIZ
    #undef  BUFSIZ
    #define BUFSIZ      8192
#endif

#define NCIDPORT    3333
#define LOCALHOST   "127.0.0.1"
#define GW          "CIDEASY"
#define TIMEOUT     2000
#define CALLS       20 /* 5 devices of 4 ports each */
#define NOPICKUP    "-"

/*
 * compiler option -Wformat-overflow gives warning for %s unless length
 * is used to reduce it from BUFSIZ (%s changed to %.275s)
 */
#define CALL        "CALL: ###DATE%.275s...CALL%.275s...LINE%.275s...NMBR%.275s...MESG%.275s...NAME%.275s+++%.275s"
#define CALLINFO    "CALLINFO: ###%.275s...DATE%.275s...SCALL%.275s...PCALL%.275s...ECALL%.275s...CALL%.275s...LINE%.275s...NMBR%.275s...NAME%.275s+++%.275s"

#define IDENT       "HELLO: IDENT: "
#define GWOUT       " [OUT]"
#define COMMAND     "HELLO: CMD: "
#define COMMAND_SIZE   8       // CIDEasy commands are 8 bytes in length

#define CRLF        "\r\n"
#define CR          '\r'
#define NL          '\n'
#define FATAL       1
#define NOTFATAL    0

#define VID          0x1234 /* KYTRONIC vendor ID. */
#define PID          0x0001 /* CID Easy product ID. */

#define MAXDEVICES   10
#define DATASIZE     512
#define GWSIZE       256
#define REPLYSIZE    8

#define DATAWAIT     2000   /* number of milliseconds to wait for data */
#define RINGWAIT     8      /* number of seconds to wait for another RING */

#define ENQ          "ENQ"
#define ETB          "ETB"

#define ACK          "ACK"
#define DCK          "DCK"
#define NCK          "NAK"

#define MDMF_CALLER_NMBR  0x02
#define MDMF_CALLER_NAME  0x07

#define CALLOUT     "OUT"
#define CALLIN      "IN"
#define BYE         "BYE"
#define CANCEL      "CANCEL"
#define CALLMWI     "MWI"

/*
 * WITHHELD    = 0x50 'P'RIVATE
 * UNAVAILABLE = 0x4F 'O'UT OF AREA
 */
#define LETTER_P    0x50
#define LETTER_O    0x4F
#define NONMBR      "NO-NUMBER"
#define NONAME      "NO NAME"

/*
 * originally:
 * O - OUT-OF-AREA
 * A - ANONYMOUS
 * P - PRIVATE
 */
#define WORDFOR_O "Unavailable"
#define WORDFOR_A "ANONYMOUS"
#define WORDFOR_P "Withheld"

/*
 * Start of XDMF message definitions
 *
 * An XDMF message starts with one of the following bytes:
 *     SDMF_MESG_TYPE
 *     MDMF_MESG_TYPE
 *     MDMF_MSG_WAITING
 *
 * MDMF_MSG_WAITING is seldom used, it indicates voicemail
 * https://en.wikipedia.org/wiki/Message-waiting_indicator
 */

#define SDMF_MESG_TYPE    0x04
#define MDMF_MESG_TYPE    0x80
#define MDMF_MSG_WAITING  0x82

#define MDMF_VISUAL_IND   0x0B

#define MDMF_DATETIME     0x01
#define MDMF_CALLER_NMBR  0x02
#define MDMF_CALLER_NAME  0x07

#define MDMF_WHY_NO_NMBR  0x04
#define MDMF_WHY_NO_NAME  0x08

/*
 * Two reasons defined for (SDMF, MDMF) WHY_NO_NUMBER
 * and WHY_NO_NAME (MDMF)
 * WITHHELD    = 0x50   'P'RIVATE
 * UNAVAILABLE = 0x4F   'O'UT OF AREA
 */
#define LETTER_P          0x50
#define LETTER_O          0x4F

/*
 * CALL_TYPE_UK is only sent for ETSI FSK and BT FSK
 * In general it can be ignored.
 */
#define CALL_TYPE_UK      0x11
/*
 * There are three call subtypes defined:
 */
#define VOICE_UK          0x01
#define RINGBACK_UK       0x02
#define MSG_WAITING_UK    0x81

#define MDMF_NMSS_UK      0x13  /* Network Message System Status */

/* End of XDMF message definitions */

#define PRETTYINDENT  "       "

#define SHOWVER     "%s %s\n%s\n"
#define DESC        "%s - CID Easy to NCID server gateway\n"
#define NOOPT       "%s: not an option: %s\n"
#define USAGE       "\
Usage:   %s [options]\n\
Options: [-C <filename>      | --config <filename>]\n\
         [-D                 | --debug]\n\
         [-h                 | --help]\n\
         [-H <0|1>           | --hostname-flag <0|1>]\n\
         [-g <lineid <name>  | --gstewayid <name>]\n\
         [-L <filename>      | --logfile <filename>]\n\
         [-n <[host][:port]> | --ncidhost <[host][:port]>]\n\
         [-P <filename>      | --pidfile <filename>]\n\
         [-t                 | --test]\n\
         [-T <testfile>      | --test-file <filename>]\n\
         [-v <1-9>           | --verbose <1-9>]\n\
         [-V                 | --version]\n\
         [--osx-launchd]\n\
"
