/*
 * sip2ncid.h - This file is part of sip2ncid.
 *
 * Copyright (c) 2005-2021
 * by John L. Chmielewski <jlc@users.sourceforge.net>
 *
 * sip2ncid is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sip2ncid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with sip2ncid.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <getopt.h>
#include <string.h>
#include <signal.h>
#include <limits.h>
#include <netdb.h>
#include <pcap.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <net/ethernet.h>
#include <netinet/in_systm.h> /* needed for FreeBSD */
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include "version.h"
#include "libncid.h"

#ifndef HOST_NAME_MAX
#define HOST_NAME_MAX 64 /* HOST_NAME_MAX is POSIX.1, Linux defines it as 64 */
#endif

#define SHOWVER     "%s %s\n%s\n"
#define DESC        "%s - Inject CID info by snooping SIP 'INVITES'\n"
#define NOOPT       "%s: not an option: %s\n"
#define USAGE       "\
Usage:   %s [options]\n\
Options: [-C <filename>      | --config <filename>]\n\
         [-D                 | --debug]\n\
         [-h                 | --help]\n\
         [-H <0|1>           | --hostname-flag <0|1>]\n\
         [-i <interface>     | --interface <interface>]\n\
         [-l                 | --listdevs]\n\
         [-L <filename>      | --logfile <filename>]\n\
         [-n <[host][:port]> | --ncid <[host][:port]>]\n\
         [-P <filename>      | --pidfile <filename>]\n\
         [-r <dumpfile>      | --readfile <dumpfile>]\n\
         [-s <[host][:port]> | --sip <[host][:port]>]\n\
         [-S <1-9>           | --lineid-size <1-9>]\n\
         [-t                 | --testsip]\n\
         [-T                 | --testall]\n\
         [-v <1-9>           | --verbose <1-9>]\n\
         [-V                 | --version]\n\
         [-w <dumpfile>      | --writefile <dumpfile>]\n\
         [-W <0|1>           | --warn <0|1>]\n\
         [--osx-launchd]\n\
"

/* make sure BUFSIZ on non-Linux is the same as on Linux */
#ifndef _G_BUFSIZ
#undef  BUFSIZ
#define BUFSIZ      8192
#endif

#define NCIDPORT    3333
#define SIPPORT     10000
#define LOCALHOST   "127.0.0.1"

#define MAXLINE     10
#define FATAL       1
#define NOTFATAL    0

#define MAXCALL     30
#define CIDSIZ      75
#define PKTWAIT     120

#define IDENT       "HELLO: IDENT: "
#define GWOUT       " [OUT]"
#define COMMAND     "HELLO: CMD: "

#define PROTOUDP    "UDP"
#define PROTOTCP    "TCP"


/* pcap_open_live(): wait after packet received in ms */
#define PCAPWAIT    1

#ifndef PIDFILE
#define PIDFILE     "/var/run/sip2ncid.pid"
#endif

#ifndef LOGFILE
#define LOGFILE     "/var/log/sip2ncid.log"
#endif

#define NODESC      "No description available"
#define LOOPBACK    "Loopback device"
#define VIRBR       "Virtual bridge"

/* strings to search for in packet */
#define SIPVER      "SIP/2"
#define REGISTER    "REGISTER"
#define INVITE      "INVITE"
#define CANCEL      "CANCEL"
#define NOTIFY      "NOTIFY"
#define BUSY        "BUSY"
#define UNKNOWN     "UNKNOWN"
#define ACK         "ACK"
#define BYE         "BYE"
#define OK          "OK"
#define REQBUSY     "486"
#define REQTERM     "487"
#define SIPNUM      "<sip:"
#define SIPCOLON    "sip:"
#define SIPAT       '@'
#define SIPTAG      ";tag="
#define FROM        "From:"
#define TO          "To:"
#define CALLID      "Call-ID:"
#define OUTCALL     "Call-ID: call-"
#define CONTACT     "Contact:"
#define PROXY       "Proxy-Authenticate:"
#define AGENT       "User-Agent:"
#define SERVER      "Server:"
#define MESSAGES    "Message-Waiting:"
#define MESSAGES_ON "Messages-Waiting: yes"
#define MESSAGES_OFF "Messages-Waiting: no"
#define QUOTE       '"'
#define NL          '\n'

#define NONAME      "NO NAME"
#define BADNAME     "BAD NAME"
#define MWINUMBER   "Voicemail"
#define MWINAME_ON  "Message(s) Waiting"
#define MWINAME_OFF "No Messages Waiting"
#define NONUMBER    "NO-NUMBER"
#define BADNUMBER   "BAD-NUMBER"
#define CALLOUT     "OUT"
#define CALLIN      "IN"
#define CALLMWI     "MWI"

#define REGLINE     "Registered: phoneline[%d] = %s\n"
/* compiler option -Wformat-overflow gives warning for %s unless length
 * is used to reduce it from BUFSIZ (%s changed to %.275s)
 */
#define CALL        "CALL: ###DATE%.275s...CALL%.275s...LINE%.275s...NMBR%.275s...NAME%.275s+++\r\n"
#define CALLINFO    "CALLINFO: ###%.275s...DATE%.275s...SCALL%.275s...ECALL%.275s...CALL%.275s...LINE%.275s...NMBR%.275s...NAME%.275s+++\r\n"

#define EQUALS      "==============================================================================="
