#!/bin/bash -e

usage="usage: ./create_32-txt.sh [--clobber-everything | --make-everything]"

# --clobber-everything will delete all metadata files that are
# downloaded and/or created, as well as libphonenumber source files.

# --make-everything will do the same as --clobber-everything and then
# force them to be downloaded and/or created.

# --make-everything and --clobber-everything should be passed to
# subsequent scripts that create files.

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

download_if_havent_yet() {

   # pass <url> [<optional filename>]
   # as long as the filename is the last part of the url, the
   # <optional filename> does not need to be passed

   if [ $# -ne 1 ] && [ $# -ne 2 ] ; then
      echo "Function '${FUNCNAME[0]}' expects one or two arguments but was passed $#."
      abort
   fi

   url="$1"
   filename_only=`basename "$url"`
   [ -n "$2" ] && filename_only=$2

   if [ ! -f $filename_only ] ; then
      echo "Downloading '$filename_only'..."
      wget2 --no-verbose -N --no-hsts   --force-progress "$url"
      rc=$?
      if [ $rc -ne 0 ] || [ ! -f $filename_only ] ; then
         echo "Could not download '$url'"
         echo "Maybe it was moved? See README-`pwd` for more info."
         abort -1
      fi
   fi

   return 0

}

abort_if_file_not_found_or_zero_bytes() {

   # pass as many filenames as desired to $1, $2, $3, etc.

   [ $# -eq 0 ] && echo "Function '${FUNCNAME[0]}' needs one or more filenames but wasn't passed any." && abort

   abort_flag=0

   for ifile in $@
      do
        if [ ! -f "$ifile" ] ; then
           echo "File '$ifile' was not found."
           abort_flag=1
        elif [ ! -s "$ifile" ] ; then
           echo "File '$ifile' was created but has zero bytes."
           abort_flag=1
        fi
   done

   [ $abort_flag -eq 1 ] && abort

   return 0

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

clobber_or_make_arg="$1"

action=GENERATING
if [ $# -gt 1 ] ; then
   echo "Too many arguments passed to $this_script."
   echo "$usage"
   abort
elif [ $# -eq 1 ] ; then
   clobber_or_make_arg="$1"
   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      action=CLOBBERING
   elif [ $clobber_or_make_arg = "--make-everything" ] ; then
      action=$action # do nothing
   else
      echo "Invalid argument passed to $this_script."
      echo "$usage"
      abort
   fi
fi

title="${action} BELGIAN METADATA"
dashes=`echo "$title" | tr '[:print:]' '-'`
echo
echo "$dashes"
echo "$title"
echo "$dashes"
echo

result_file=32.txt

# tmp1=${this_script_base_no_ext}-1.tmp
# rm -f *.tmp

rm -f ${result_file}*  *.csv  file1 current_database_url

if [ -n "$clobber_or_make_arg" ] ; then

   rm -f *.xlsx 

   [ $clobber_or_make_arg = "--clobber-everything" ] && exit 0

fi

# check for required directories and/or scripts
   abort_flag=0

   cd ..
   metadata_generation_dir=`pwd` # saves full path

   # make sure libphonenumber is installed
   x=${metadata_generation_dir}/resources/carrier
        if [ ! -d $x ] ; then
           echo "Required directory '$x' not found."
           echo "This means 'libphonenumber' does not appear to be installed."
           echo "See ${metadata_generation_dir}/README-`basename ${metadata_generation_dir}` for more info."
           abort_flag=1
        fi

   # make sure we can find ssconvert or xlsx2csv
		if ( ! which ssconvert >/dev/null 2>&1  && ! which xlsx2csv >/dev/null 2>&1 ) ; then
           echo "Required program 'ssconvert' or 'xlsx2csv' not found."
           abort -1
        fi

[ $abort_flag -eq 1 ] && abort

cd $this_script_dir

# this file lists the carriers and their corresponding codes

#this URL should be updated regularly ...
# refer to
# https://ibpt.be/operateurs/publication/base-de-donnees-des-numeros-reserves-et-affectes
# for exact location 
echo -n  "https://ibpt.be" >  current_database_url
wget2  --no-verbose --output-document=-  --force-progress  https://ibpt.be/operateurs/publication/base-de-donnees-des-numeros-reserves-et-affectes  |

grep  "Téléchargez le fichier&nbsp;&quot;Base de données des numéros réservés et affectés&quot" | sed -e "s/^.*href=\"\(.*\.xlsx\)\".*$/\1/ " >>  current_database_url

if grep -q ".xlsx" current_database_url
then
echo "Good !... We recovered an URL pointing to an xlsx file ..."
else 
echo "Could not retrieve the URL for the current_database of Belgium phone numbers attributions..\n Please check https://ibpt.be/operateurs/publication/base-de-donnees-des-numeros-reserves-et-affectes "
abort
fi


echo current_database_url:
cat  current_database_url

download_if_havent_yet ` cat current_database_url ` 
rm -f belgium_phone_range.xlsx
if [ -f *.xlsx ]
then 
  echo "we have just downloaded an  xlsx file: "
  ls *.xlsx
  cp *.xlsx belgium_phone_range.xlsx
else 
  echo "Could not download the current_database of Belgium phone numbers (xlsx file )" 
  abort
fi


rm -f *.csv
echo "Started conversion of official Belgian numbers xls files into csv..."
if (which ssconvert  > /dev/null  2>&1 ) 
then 
echo "converting using ssconvert..."
ssconvert -S  belgium_phone_range.xlsx %s.csv 2>/dev/null
else
  echo "no ssconvert  installed"
  if (which xlsx2csv  > /dev/null  2>&1 )
  then 
    echo "converting using xlsx2cvs..."
    xlsx2csv -s 0 belgium_phone_range.xlsx  .
  fi

fi


echo "Finished conversion of official Belgian carrier xls files into csv."

#convert annoying file with space in name ...
mv '_short sms.csv' _short_sms.csv


echo "Converting to Google metadata format..."
cat /dev/null > file1

echo "processing mobile phones..." 
for cvs_file in _4pq.csv
do
awk -F "\"*,\"*" '{ 
                   if ($2=="A" && $4 !="" )
				     print $1 "," $4
				   }'  $cvs_file  >> file1
done

echo "processing  070: Premium pay rate services"
echo "processing  077: Machine to machine communication"
echo "processing  078: National pay rate services."
for cvs_file in _07*.csv
do
awk -F "\"*,\"*" '{ 
                   if ($3=="A" && $5 !="" )
				     print $1$2 "," $5
				   }'  $cvs_file  >> file1
done

echo "processing  0800: toll free call rate services."
for cvs_file in _08*.csv
do
awk -F "\"*,\"*" '{ 
                   if ($3=="A" && $5 !="" )
				     print $1$2 "," $5
				   }'  $cvs_file  >> file1
done

echo "processing  090x - Premium pay rate numbers*"
#
#    0900 (€0.50/min.).
#    HUP	Wed 22 Nov 2023	15:33	00:00:04	04 24 23 12 43	(Carrier) DVSC Métropole			France	DVS Connect	0901 (€0.50/call).
#    0902 (€1/min.).
#    0903 (€1.50/min.).
#    0904 (€2/min.).
#    0905 (€2/call).
#    0906 (€1/min.).
#    0907 (€2/min.).
#    0909 (€31/call).
for cvs_file in _09*.csv
do
awk -F "\"*,\"*" '{ 
                   if ($3=="A" && $5 !="" )
				     print $2 "," $5
				   }'  $cvs_file  >> file1
done

echo "processing  Landlines (geographical numbers)"
awk -F "\"*,\"*" '{ 
                   if ($3=="A" && $4 !="" )
				     print $2 "," $4
				   }'   PQYZ.csv >> file1

echo "processing  Short SMS numbers" 

awk -F "\"*,\"*" '{
                   if ($2=="A" && $4 !="" )
                     print $1 "," $4
                   }'  _short_sms.csv  >> file1

abort_if_file_not_found_or_zero_bytes file1
    sed -E "s/^0//" file1 | sed -E "s/^/32/" |
    sed -e "s/Proximus SA de droit public/Proximus/"|sort -n |sed -E "s/,/|/" > ${result_file}_new
rm -f file1
# be careful (replacing "," by "|" is done after  sorting (not to disturb
# sorting order ) 

abort_if_file_not_found_or_zero_bytes ${result_file}_new

cat libphonenumber-and-ncid-comments.txt ${result_file}_new > $result_file
rm -f ${result_file}_new
check_error
abort_if_file_not_found_or_zero_bytes $result_file

echo "Resulting file '$result_file' has `cat $result_file |wc -l` lines"

echo "Copying '$result_file' to ${metadata_generation_dir}/resources/carrier/en/..."
cp -p $result_file ${metadata_generation_dir}/resources/carrier/en/
check_error

echo "Script completed successfully."

# all conversion scripts need to do 'exit 0' if successful so that their calling script (if any) will not abort
exit 0
