#!/bin/bash -e

usage="usage: ./create_33-txt.sh [--clobber-everything | --make-everything]"

# --clobber-everything will delete all metadata files that are
# downloaded and/or created, as well as libphonenumber source files.

# --make-everything will do the same as --clobber-everything and then
# force them to be downloaded and/or created.

# --make-everything and --clobber-everything should be passed to
# subsequent scripts that create files.

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

download_if_havent_yet() {

   # pass <url> [<optional filename>]
   # as long as the filename is the last part of the url, the
   # <optional filename> does not need to be passed

   if [ $# -ne 1 ] && [ $# -ne 2 ] ; then
      echo "Function '${FUNCNAME[0]}' expects one or two arguments but was passed $#."
      abort
   fi

   url="$1"
   filename_only=`basename "$url"`
   [ -n "$2" ] && filename_only=$2

   if [ ! -f $filename_only ] ; then
      echo "Downloading '$filename_only'..."
      wget2 --no-verbose -N --no-hsts   --force-progress "$url"
      rc=$?
      if [ $rc -ne 0 ] || [ ! -f $filename_only ] ; then
         echo "Could not download '$url'"
         echo "Maybe it was moved? See README-`pwd` for more info."
         abort -1
      fi
   fi

   return 0

}

abort_if_file_not_found_or_zero_bytes() {

   # pass as many filenames as desired to $1, $2, $3, etc.

   [ $# -eq 0 ] && echo "Function '${FUNCNAME[0]}' needs one or more filenames but wasn't passed any." && abort

   abort_flag=0

   for ifile in $@
      do
        if [ ! -f "$ifile" ] ; then
           echo "File '$ifile' was not found."
           abort_flag=1
        elif [ ! -s "$ifile" ] ; then
           echo "File '$ifile' was created but has zero bytes."
           abort_flag=1
        fi
   done

   [ $abort_flag -eq 1 ] && abort

   return 0

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

clobber_or_make_arg="$1"

action=GENERATING
if [ $# -gt 1 ] ; then
   echo "Too many arguments passed to $this_script."
   echo "$usage"
   abort
elif [ $# -eq 1 ] ; then
   clobber_or_make_arg="$1"
   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      action=CLOBBERING
   elif [ $clobber_or_make_arg = "--make-everything" ] ; then
      action=$action # do nothing
   else
      echo "Invalid argument passed to $this_script."
      echo "$usage"
      abort
   fi
fi

title="${action} FRENCH METADATA"
dashes=`echo "$title" | tr '[:print:]' '-'`
echo
echo "$dashes"
echo "$title"
echo "$dashes"
echo

result_file=33.txt

# tmp1=${this_script_base_no_ext}-1.tmp
# rm -f *.tmp

rm -f ${result_file}* *.csv file1

if [ -n "$clobber_or_make_arg" ] ; then

   rm -f MAJOPE.xls MAJNUM.xls

   [ $clobber_or_make_arg = "--clobber-everything" ] && exit 0

fi

# check for required directories and/or scripts
   abort_flag=0

   cd ..
   metadata_generation_dir=`pwd` # saves full path

   # make sure libphonenumber is installed
   x=${metadata_generation_dir}/resources/carrier
        if [ ! -d $x ] ; then
           echo "Required directory '$x' not found."
           echo "This means 'libphonenumber' does not appear to be installed."
           echo "See ${metadata_generation_dir}/README-`basename ${metadata_generation_dir}` for more info."
           abort_flag=1
        fi

   # make sure we can find xls2csv.sh
   x=${metadata_generation_dir}/xls2csv.sh
        if [ ! -x $x ] ; then
           echo "Required script '$x' not found."
           abort_flag=1
        fi

[ $abort_flag -eq 1 ] && abort

cd $this_script_dir

# this file lists the carriers and their corresponding codes

#commented out by BGR 18/02/2023 (dead URL)
#download_if_havent_yet https://extranet.arcep.fr/portail/Portals/0/Documents/Arcep/Num%C3%A9rotation/MAJOPE.xls

download_if_havent_yet https://www.data.gouv.fr/fr/datasets/r/b0f62183-cd0c-498d-8153-aa1594e5e8d9
# this is supposed to be a "permanent" "open data" government URL
#(Current official name of the downloaded file:    identifiants_CE.csv)
#(used to be MAJOPE.xls before (with a different format))

#This file contains the various phone number ranges and the carrier code they have beeen attributed to
#download_if_havent_yet https://extranet.arcep.fr/portail/Portals/0/Documents/Arcep/Num%C3%A9rotation/MAJNUM.xls
# 17 Sept 2024 , yet again they did move the file around ....
download_if_havent_yet https://www.data.gouv.fr/fr/datasets/r/90e8bdd0-0f5c-47ac-bd39-5f46463eb806
# name is now  MAJNUM.csv 

ls -l 90e8bdd0-0f5c-47ac-bd39-5f46463eb806

echo "Downloaded file ...\n"

mv 90e8bdd0-0f5c-47ac-bd39-5f46463eb806 MAJNUM.csv
ls -l MAJNUM.csv

#echo "Started conversion of official French numbers xls files into csv..."
#${metadata_generation_dir}/xls2csv.sh  MAJNUM.xls
# 17 Sept 2024 --> no need to convert now , as file is already in csv format . 

mv b0f62183-cd0c-498d-8153-aa1594e5e8d9  identifiants_CE.csv
if [ ! `isutf8 -q identifiants_CE.csv` ] 
then
  echo "Converting identifiants_CE.csv from ISO-8859-1 to  UTF-8."
  cp identifiants_CE.csv identifiants_CE.csv_tmp
  iconv -f ISO-8859-1 -t UTF-8 identifiants_CE.csv_tmp -o identifiants_CE.csv
  rm identifiants_CE.csv_tmp
fi
awk -F "\"*;\"*" '{print $2 "," $1}' identifiants_CE.csv > MAJOPE.csv

echo "Converting to Google metadata format..."

# 17 Sept 2024 -->separator changed from ','  to ';' for MAJNUM.csv
awk -F "\"*;\"*" '{print $1 "," $4}' MAJNUM.csv  | grep "^0" >file1

abort_if_file_not_found_or_zero_bytes file1

awk -F"[,]" 'NR==FNR{a[$1]=$2","$2; next} ($2 in a){print $1","a[$2]}'  MAJOPE.csv file1 |
    awk -F "\"*,\"*" '{print $1"|"$2}' |
    sort |
    sed -E "s/^0/33/" |
    sed -e "s/Société française du radiotéléphone/SFR/" |
    sed -e "s/Société réunionnaise du radiotéléphone/SFR Réunion/" > ${result_file}_new

abort_if_file_not_found_or_zero_bytes ${result_file}_new

cat libphonenumber-and-ncid-comments.txt ${result_file}_new > $result_file
check_error
abort_if_file_not_found_or_zero_bytes $result_file

echo "Resulting file '$result_file' has `cat $result_file |wc -l` lines"

echo "Copying '$result_file' to ${metadata_generation_dir}/resources/carrier/en/..."
cp -p $result_file ${metadata_generation_dir}/resources/carrier/en/
check_error

echo "Script completed successfully."

# all conversion scripts need to do 'exit 0' if successful so that their calling script (if any) will not abort
exit 0
