#!/bin/bash -e
# Last time modified: Sep 19, 2024

usage="usage: ./metadata_generation.sh [--clobber-everything | --make-everything]"

# --clobber-everything will delete all metadata files that are
# downloaded and/or created, as well as libphonenumber source files.

# --make-everything will do the same as --clobber-everything and then
# force them to be downloaded and/or created.

# --make-everything and --clobber-everything should be passed to
# subsequent scripts that create files.

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

download_if_havent_yet() {

   # pass <url> [<optional filename>]
   # as long as the filename is the last part of the url, the
   # <optional filename> does not need to be passed

   if [ $# -ne 1 ] && [ $# -ne 2 ] ; then
      echo "Function '${FUNCNAME[0]}' expects one or two arguments but was passed $#."
      abort
   fi

   url="$1"
   filename_only=`basename "$url"`
   [ -n "$2" ] && filename_only=$2

   if [ ! -f $filename_only ] ; then
      echo "Downloading '$filename_only'..."
      wget2 -N --no-verbose --no-hsts --force-progress "$url"
      rc=$?
      if [ $rc -ne 0 ] || [ ! -f $filename_only ] ; then
         echo "Could not download '$url'"
         echo "Maybe it was moved? See README-`pwd` for more info."
         abort -1
      fi
   fi

   return 0

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

echo "Executing script '${this_script_dir}/${this_script_base}'..."

clobber_or_make_arg="$1"

action=
if [ $# -gt 1 ] ; then
   echo "Too many arguments passed to $this_script."
   echo "$usage"
   abort
elif [ $# -eq 1 ] ; then
   clobber_or_make_arg="$1"
   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      action=CLOBBERING
   elif [ $clobber_or_make_arg = "--make-everything" ] ; then
      action=GENERATING
   else
      echo "Invalid argument passed to $this_script."
      echo "$usage"
      abort
   fi
fi

if [ -z "$action" ] ; then
   title="CREATING CARRIER_DATA.CC"
else
   title="${action} EVERYTHING"
fi

dashes=`echo "$title" | tr '[:print:]' '='`
echo
echo "$dashes"
echo "$title"
echo "$dashes"
echo

# List all top-level data generation scripts here. Order is not important.
scripts=" \
          ${this_script_dir}/nanpa_data_generation/create_1-txt.sh \
          ${this_script_dir}/fr_data_generation/create_33-txt.sh   \
          ${this_script_dir}/uk_data_generation/create_44-txt.sh   \
          ${this_script_dir}/jp_data_generation/create_81-txt.sh   \
          ${this_script_dir}/be_data_generation/create_32-txt.sh   \
        "

# check for required directories and/or scripts

  abort_flag=0

  cd $this_script_dir
  x=../libcarrier
       if [ ! -d $x ] ; then
          echo "Required directory '$x' not found."
          abort_flag=1
       else
          cd $x
          libcarrier_dir=`pwd` # saves full path
       fi

  cd $this_script_dir
  x=./carrier_data_generator
       if [ ! -d $x ] ; then
          echo "Required directory '$x' not found."
          abort_flag=1
       else
          cd $x
          carrier_data_dir=`pwd` # saves full path
       fi

  # make sure we can find xls2csv.sh
  x=${this_script_dir}/xls2csv.sh
       if [ ! -x $x ] ; then
          echo "Required script '$x' not found."
          abort_flag=1
       fi
   # make sure we can find ssconvert or xlsx2csv
	   if ( ! which ssconvert >/dev/null 2>&1  && ! which xlsx2csv >/dev/null 2>&1 ) ; then
          echo "Required program 'ssconvert' or 'xlsx2csv' not found."
          abort -1
       fi
  [ $abort_flag -eq 1 ] && abort

# tmp1=${this_script_base_no_ext}-1.tmp
# rm -f *.tmp

rm -f ${this_script_dir}/carrier_data.cc

if [ -n "$clobber_or_make_arg" ] ; then

   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then

      rm -fr ${this_script_dir}/resources ${this_script_dir}/tmp

      cd $carrier_data_dir
         make clobber
         check_error

      for runscript in $scripts
         do
           echo
           echo
           echo "Executing script '$runscript'..."
           [ ! -x $runscript ] && echo "Could not find and/or execute script '$runscript'" && abort
           cd `dirname $runscript`
           $runscript $clobber_or_make_arg
           check_error
      done

      title="${action} ALL DONE"
      dashes=`echo "$title" | tr '[:print:]' '='`
      echo
      echo "$dashes"
      echo "$title"
      echo "$dashes"
      echo

      exit 0

   fi

fi

xls2csv_helper=`${this_script_dir}/xls2csv.sh findprog` # used to prevent checking which program is installed for every conversion
check_error
export xls2csv_helper

# check if we have libphonenumber
  if [ ! -d ${this_script_dir}/resources/carrier ] ; then

     mkdir -p ${this_script_dir}/tmp
     check_error
     cd ${this_script_dir}/tmp

     # download libphonenumber and extract
       echo "Downloading 'libphonenumber' to `pwd`..."

       # taa from an email BRG wrote to JLC:
       #     On Mon, Jun 15, 2020 at 07:28:14AM +0200, grld wrote:
       #     ncid is atm compiled to work with libphonenumber 8.12.4 (from memory )

       # two options available
       # 1. get a specific version
       #    release_url=https://github.com/google/libphonenumber/archive/v8.12.4.tar.gz
       #
       # 2. get the latest official release
       #    latest_version=`wget2 --no-verbose --no-hsts --force-progress -q -O - \
       #                         https://api.github.com/repos/google/libphonenumber/releases/latest |
       #                         grep tag_name |
       #                         sed "s/\"//g;s/,$//" |
       #                         awk '{print $2}'`
       #    release_url=https://github.com/google/libphonenumber/archive/${latest_version}.tar.gz
       rm -f /tmp/latest
	   latest_version=""
       wget2 --no-verbose --no-hsts --force-progress -q -O /tmp/latest  https://api.github.com/repos/google/libphonenumber/releases/latest
       latest_version=`grep tag_name /tmp/latest |
							sed "s/^.*\"tag_name\":\(\"[^\"]*\"\).*$/\1/" |
                            sed "s/\"//g;s/,$//" `
       rm -f /tmp/latest
	   echo "libphonenumber latest version: $latest_version"
	   if [ $latest_version == "" ]
	   then
         echo " could not find latest version number for ibphonenumber \n"
		 exit 1
	   fi
       release_url=https://github.com/google/libphonenumber/archive/${latest_version}.tar.gz

       release_file=`basename $release_url`
       download_if_havent_yet $release_url

       echo "Extracting..."
       tar xzf $release_file
       check_error

       cd libphone*
       downloaded_libphone_dir=`pwd`

       x=${downloaded_libphone_dir}/release_notes.txt
       [ ! -f $x ] && echo "Could not find file '$x'" && abort
       echo "Successfully extracted libphonenumber version `head -1 $x|awk '{print $NF}'`"

       echo "Moving downloaded resource files to '${this_script_dir}/resources/carrier'..."
          mkdir -p ${this_script_dir}/resources/carrier
          check_error
          mv resources/carrier/* ${this_script_dir}/resources/carrier/
          check_error

  fi

# we have libphonenumber loaded -- now create metadata files
cd $this_script_dir

echo "Creating country files..."
for runscript in $scripts
    do
      echo
      echo
      echo "Executing script '$runscript'..."
      [ ! -x $runscript ] && echo "Could not find and/or execute script '$runscript'" && abort
      cd `dirname $runscript`
      $runscript $clobber_or_make_arg
      check_error
done

cd $this_script_dir

prog=$carrier_data_dir/generate_carrier_data
if [ ! -x $prog ]; then
   echo "Compiling carrier data generator tool..."
   # create carrier_data_generator/generate_carrier_data
   cd $carrier_data_dir
   make clobber all
   check_error
   cd $this_script_dir
fi

echo "Compile resources directory into carrier_data.cc..."

$prog ${this_script_dir}/resources/carrier ${this_script_dir}/carrier_data.cc
check_error

x=${this_script_dir}/carrier_data.cc
[ ! -f $x ] && echo "Could not find file '$x'" && abort

echo
echo "Operation successful!"
size=`ls -l carrier_data.cc`
sizeH=`ls -lh carrier_data.cc | sed 's/^.* \([0-9]*[A-Z]\) .*$/\1/'`
echo "$size ($sizeH)"
echo
echo "Current libcarrier/carrier_data.cc:"
lsize=`ls -l ../libcarrier/carrier_data.cc`
lsizeH=`ls -lh ../libcarrier/carrier_data.cc | sed 's/^.* \([0-9]*[A-Z]\) .*$/\1/'`
echo "$lsize ($lsizeH)"
echo
echo "If sizes look good, copy carrier_data.cc to libcarrier: make update"
