#!/bin/bash -e

usage="usage: ./canada_data_script.sh [--clobber-everything | --make-everything]"

# --clobber-everything will delete all metadata files that are
# downloaded and/or created, as well as libphonenumber source files.

# --make-everything will do the same as --clobber-everything and then
# force them to be downloaded and/or created.

# --make-everything and --clobber-everything should be passed to
# subsequent scripts that create files.

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

download_if_havent_yet() {

   # pass <url> [<optional filename>]
   # as long as the filename is the last part of the url, the
   # <optional filename> does not need to be passed

   if [ $# -ne 1 ] && [ $# -ne 2 ] ; then
      echo "Function '${FUNCNAME[0]}' expects one or two arguments but was passed $#."
      abort
   fi

   url="$1"
   filename_only=`basename "$url"`
   [ -n "$2" ] && filename_only=$2

   if [ ! -f $filename_only ] ; then
      echo "Downloading '$filename_only'..."
      wget2 -N --no-verbose --no-hsts --force-progress "$url"
      rc=$?
      if [ $rc -ne 0 ] || [ ! -f $filename_only ] ; then
         echo "Could not download '$url'"
         echo "Maybe it was moved? See README-`pwd` for more info."
         abort -1
      fi
   fi

   return 0

}

abort_if_file_not_found_or_zero_bytes() {

   [ $# -eq 0 ] && echo "Function '${FUNCNAME[0]}' needs one or more filenames but wasn't passed any." && abort

   abort_flag=0

   for ifile in $@
      do
        if [ ! -f "$ifile" ] ; then
           echo "File '$ifile' was not found."
           abort_flag=1
        elif [ ! -s "$ifile" ] ; then
           echo "File '$ifile' was created but has zero bytes."
           abort_flag=1
        fi
   done

   [ $abort_flag -eq 1 ] && abort

   return 0

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

clobber_or_make_arg="$1"

action=GENERATING
if [ $# -gt 1 ] ; then
   echo "Too many arguments passed to $this_script."
   echo "$usage"
   abort
elif [ $# -eq 1 ] ; then
   clobber_or_make_arg="$1"
   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      action=CLOBBERING
   elif [ $clobber_or_make_arg = "--make-everything" ] ; then
      action=$action # do nothing
   else
      echo "Invalid argument passed to $this_script."
      echo "$usage"
      abort
   fi
fi

#title="${action} ??? METADATA"
#dashes=`echo "$title" | tr '[:print:]' '-'`
#echo
#echo "$dashes"
#echo "$title"
#echo "$dashes"
#echo

result_file=1_can.txt

# tmp1=${this_script_base_no_ext}-1.tmp
# rm -f *.tmp 

rm -f ${result_file}

if [ -n "$clobber_or_make_arg" ] ; then

   rm -f COCodeStatus_ALL.zip

   [ $clobber_or_make_arg = "--clobber-everything" ] && exit 0

fi

download_if_havent_yet http://www.cnac.ca/data/COCodeStatus_ALL.zip

echo "Started extracting Canada data from 'COCodeStatus_ALL.zip'..."

unzip -p  COCodeStatus_ALL.zip |
    grep  ",\"In Service\"," |
    awk  -F","  '{print $1 $2 "|" $6}' |
    sed -e "s/\"//g"|
    sed -e "s/^/1/" > $result_file

abort_if_file_not_found_or_zero_bytes $result_file

echo "Resulting file '$result_file' has `cat $result_file |wc -l` lines"

echo "Script completed successfully."

# all conversion scripts need to do 'exit 0' if successful so that their calling script (if any) will not abort
exit 0
