#!/bin/bash -e

usage="usage: ./create_1-txt.sh [--clobber-everything | --make-everything]"

# --clobber-everything will delete all metadata files that are
# downloaded and/or created, as well as libphonenumber source files.

# --make-everything will do the same as --clobber-everything and then
# force them to be downloaded and/or created.

# --make-everything and --clobber-everything should be passed to
# subsequent scripts that create files.

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

abort_if_file_not_found_or_zero_bytes() {

   [ $# -eq 0 ] && echo "Function '${FUNCNAME[0]}' needs one or more filenames but wasn't passed any." && abort

   abort_flag=0

   for ifile in $@
      do
        if [ ! -f "$ifile" ] ; then
           echo "File '$ifile' was not found."
           abort_flag=1
        elif [ ! -s "$ifile" ] ; then
           echo "File '$ifile' was created but has zero bytes."
           abort_flag=1
        fi
   done

   [ $abort_flag -eq 1 ] && abort

   return 0

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

clobber_or_make_arg="$1"

action=GENERATING
if [ $# -gt 1 ] ; then
   echo "Too many arguments passed to $this_script."
   echo "$usage"
   abort
elif [ $# -eq 1 ] ; then
   clobber_or_make_arg="$1"
   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      action=CLOBBERING
   elif [ $clobber_or_make_arg = "--make-everything" ] ; then
      action=$action # do nothing
   else
      echo "Invalid argument passed to $this_script."
      echo "$usage"
      abort
   fi
fi

title="${action} US AND CANADA (NANPA) METADATA"
dashes=`echo "$title" | tr '[:print:]' '-'`
echo
echo "$dashes"
echo "$title"
echo "$dashes"
echo

result_file=1.txt
google_original_file=1_google_original.txt

# tmp1=${this_script_base_no_ext}-1.tmp
# rm -f *.tmp 

# List all sub scripts here. Order is not important.
scripts=" \
          ${this_script_dir}/us_data_script.sh \
          ${this_script_dir}/canada_data_script.sh   \
        "

rm -f 1_google_original_data_only.txt 1_google_original_comments.txt 1_data.txt

# don't delete with trailing asterisk ${result_file}* or you'll wipe out $google_original_file!!!!
rm -f ${result_file}

if [ -n "$clobber_or_make_arg" ] ; then

   rm -f $google_original_file

   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      for runscript in $scripts
         do
           echo
           echo
           echo "Executing script '$runscript'..."
           [ ! -x $runscript ] && echo "Could not find and/or execute script '$runscript'" && abort
           cd `dirname $runscript`
           $runscript $clobber_or_make_arg
           check_error
      done
      exit 0
   fi

fi

# check for required directories and/or scripts
   abort_flag=0

   cd ..
   metadata_generation_dir=`pwd` # saves full path

   # make sure libphonenumber is installed
   x=${metadata_generation_dir}/resources/carrier
        if [ ! -d $x ] ; then
           echo "Required directory '$x' not found."
           echo "This means 'libphonenumber' does not appear to be installed."
           echo "See ${metadata_generation_dir}/README-`basename ${metadata_generation_dir}` for more info."
           abort_flag=1
        fi

[ $abort_flag -eq 1 ] && abort

cd $this_script_dir

if [ ! -f $google_original_file ] ; then
   echo "Making a copy of the original Google 1.txt file for safekeeping..."
   x=${metadata_generation_dir}/resources/carrier/en/1.txt
   [ ! -f $x ] && echo "Could not find file '$x'" && abort
   cp -p $x $google_original_file
   check_error
fi

echo "Creating NANPA country files..."
for runscript in $scripts
    do
      echo
      echo
      echo "Executing script '$runscript'..."
      [ ! -x $runscript ] && echo "Could not find and/or execute script '$runscript'" && abort
      cd `dirname $runscript`
      $runscript $clobber_or_make_arg
      check_error
done

cd $this_script_dir

grep    "^[0-9]" $google_original_file > 1_google_original_data_only.txt
grep -v "^[0-9]" $google_original_file > 1_google_original_comments.txt

abort_if_file_not_found_or_zero_bytes 1_google_original_data_only.txt 1_google_original_comments.txt

echo "Concatenate and sort US and Canada with original Google data..."

cat 1_google_original_data_only.txt 1_us.txt 1_can.txt | sort > 1_data.txt
check_error
abort_if_file_not_found_or_zero_bytes 1_data.txt

cat 1_google_original_comments.txt libphonenumber-and-ncid-comments.txt 1_data.txt > $result_file
check_error
abort_if_file_not_found_or_zero_bytes $result_file

echo "Resulting file '$result_file' has `cat $result_file |wc -l` lines"

echo "Copying '$result_file' to ${metadata_generation_dir}/resources/carrier/en/..."
cp -p $result_file ${metadata_generation_dir}/resources/carrier/en/
check_error

echo "Script completed successfully."

# all conversion scripts need to do 'exit 0' if successful so that their calling script (if any) will not abort
exit 0
