#!/bin/bash -e

usage="usage: ./create_44-txt.sh [--clobber-everything | --make-everything]"

# --clobber-everything will delete all metadata files that are
# downloaded and/or created, as well as libphonenumber source files.

# --make-everything will do the same as --clobber-everything and then
# force them to be downloaded and/or created.

# --make-everything and --clobber-everything should be passed to
# subsequent scripts that create files.

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

download_if_havent_yet() {

   # pass <url> [<optional filename>]
   # as long as the filename is the last part of the url, the
   # <optional filename> does not need to be passed

   if [ $# -ne 1 ] && [ $# -ne 2 ] ; then
      echo "Function '${FUNCNAME[0]}' expects one or two arguments but was passed $#."
      abort
   fi

   url="$1"
   filename_only=`basename "$url"`
   [ -n "$2" ] && filename_only=$2

   if [ ! -f $filename_only ] ; then
      echo "Downloading '$filename_only'..."
      wget2 -N --no-verbose --no-hsts --force-progress "$url"
      rc=$?
      if [ $rc -ne 0 ] || [ ! -f $filename_only ] ; then
         echo "Could not download '$url'"
         echo "Maybe it was moved? See README-`pwd` for more info."
         abort -1
      fi
   fi

   return 0

}

abort_if_file_not_found_or_zero_bytes() {

   # pass as many filenames as desired to $1, $2, $3, etc.

   [ $# -eq 0 ] && echo "Function '${FUNCNAME[0]}' needs one or more filenames but wasn't passed any." && abort

   abort_flag=0

   for ifile in $@
      do
        if [ ! -f "$ifile" ] ; then
           echo "File '$ifile' was not found."
           abort_flag=1
        elif [ ! -s "$ifile" ] ; then
           echo "File '$ifile' was created but has zero bytes."
           abort_flag=1
        fi
   done

   [ $abort_flag -eq 1 ] && abort

   return 0

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

clobber_or_make_arg="$1"

action=GENERATING
if [ $# -gt 1 ] ; then
   echo "Too many arguments passed to $this_script."
   echo "$usage"
   abort
elif [ $# -eq 1 ] ; then
   clobber_or_make_arg="$1"
   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      action=CLOBBERING
   elif [ $clobber_or_make_arg = "--make-everything" ] ; then
      action=$action # do nothing
   else
      echo "Invalid argument passed to $this_script."
      echo "$usage"
      abort
   fi
fi

title="${action} UK METADATA"
dashes=`echo "$title" | tr '[:print:]' '-'`
echo
echo "$dashes"
echo "$title"
echo "$dashes"
echo

result_file=44.txt

# tmp1=${this_script_base_no_ext}-1.tmp
# rm -f *.tmp 

rm -f ${result_file}* uk_carriers.csv

if [ -n "$clobber_or_make_arg" ] ; then

   rm -f *.csv

   [ $clobber_or_make_arg = "--clobber-everything" ] && exit 0

fi

# check for required directories and/or scripts
   abort_flag=0

   cd ..
   metadata_generation_dir=`pwd` # saves full path

   # make sure libphonenumber is installed
   x=${metadata_generation_dir}/resources/carrier
        if [ ! -d $x ] ; then
           echo "Required directory '$x' not found."
           echo "This means 'libphonenumber' does not appear to be installed."
           echo "See ${metadata_generation_dir}/README-`basename ${metadata_generation_dir}` for more info."
           abort_flag=1
        fi

   # make sure we can find xls2csv.sh
   x=${metadata_generation_dir}/xls2csv.sh
        if [ ! -x $x ] ; then
           echo "Required script '$x' not found."
           abort_flag=1
        fi

[ $abort_flag -eq 1 ] && abort

cd $this_script_dir

download_if_havent_yet http://static.ofcom.org.uk/static/numbering/S1.csv
download_if_havent_yet http://static.ofcom.org.uk/static/numbering/S3.csv
download_if_havent_yet http://static.ofcom.org.uk/static/numbering/S5.csv
download_if_havent_yet http://static.ofcom.org.uk/static/numbering/S7.csv
download_if_havent_yet http://static.ofcom.org.uk/static/numbering/S8.csv
download_if_havent_yet http://static.ofcom.org.uk/static/numbering/S9.csv

echo "Converting to Google metadata format..."

cat S1.csv S3.csv S5.csv S7.csv S8.csv S9.csv > uk_carriers.csv
check_error
abort_if_file_not_found_or_zero_bytes uk_carriers.csv

grep "Allocated" uk_carriers.csv |
    awk -F "\"*,\"*" '{print $1 "|" $3}' |
    sed -E "s/(^[0-9]*) /\1/" |
    sed -E "s/^/44/" > ${result_file}_data

abort_if_file_not_found_or_zero_bytes ${result_file}_data

cat libphonenumber-and-ncid-comments.txt ${result_file}_data > $result_file
check_error
abort_if_file_not_found_or_zero_bytes $result_file

echo "Resulting file '$result_file' has `cat $result_file |wc -l` lines"

echo "Copying '$result_file' to ${metadata_generation_dir}/resources/carrier/en/..."
cp -p $result_file ${metadata_generation_dir}/resources/carrier/en/
check_error

echo "Script completed successfully."

# all conversion scripts need to do 'exit 0' if successful so that their calling script (if any) will not abort
exit 0
