#!/bin/bash -e
# script last changed on Mar 3, 2021

# usage: ./xls2csv.sh <xlsfile1> <xlsfile2> <xlsfile3> ...
#    or  ./xls2csv.sh findprog
#    or  ./xls2csv.sh xlsx
#
#        When specifying .xls file names, script will automatically set the
#        .csv file extension. The .csv file will be deleted if it already
#        exists.
#
#        Specifying the literal text 'findprog' will cause script to search
#        for known conversion programs and if found will send the name to stdout.
#        This can be saved in the exported global variable 'xls2csv_helper' to
#        prevent having to look for a conversion program every time this script
#        is called.
#

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

filetype=xls
ext=".${filetype}"
if [ "$1" = "xlsx" ] ; then
    filetype=$1
    ext=".${filetype}"
    shift
fi

found_prog=$xls2csv_helper
if [ -z "$found_prog" ] || [ "$1" = "findprog" ] ; then

   # echo output sent to stderr in case metadata_generation.sh is running
   # this script and capturing stdout

   echo "Looking for an installed program that can convert ${ext} files to '.csv'" >&2

   if [ "${ext}" = ".xls" ] ; then
      possible_prog="unoconv xls2csv libreoffice soffice"
   else
      possible_prog="unoconv xlsx2csv libreoffice soffice"
   fi
   found_prog=
   for prog in $possible_prog
   do
         if ( which $prog >/dev/null 2>&1 ) ; then
            found_prog=$prog
            echo "Found '$prog'" >&2
            break
         fi
   done

   if [ -z "$found_prog" ] ; then
      echo "No xls to cvs converter found on your system." >&2
      echo "Try installing one of these: $possible_prog" >&2
      abort -1
   else
      [ "$1" = "findprog" ] && echo "$found_prog" && exit 0
   fi
fi

for xls_file in $@
    do

      xls_file_no_ext=`basename "$xls_file" ${ext}`
      csv_file=${xls_file_no_ext}.csv
      echo "Using '$found_prog' to convert '$xls_file' to '$csv_file'..."
      [ ! -f "$xls_file" ] && echo "File '$xls_file' not found" && abort -1
      [ "$xls_file" = "$csv_file" ] && echo "Something went wrong -- the .xls and .csv file names are the same" && a    bort -1
      rm -f "$csv_file"

      case $found_prog in
                     unoconv) $found_prog -f csv -o ${csv_file} ${xls_file}
                              check_error
                              ;;
                   ssconvert) $found_prog ${xls_file}   ${csv_file}
                              check_error
                              ;;
                     xls2csv) $found_prog ${xls_file} > ${csv_file}
                              check_error
                              ;;
                    xlsx2csv) $found_prog ${xls_file} > ${csv_file}
                              check_error
                              ;;
         libreoffice|soffice) $found_prog --headless --convert-to csv:"Text - txt - csv (StarCalc)":44,34,76 ${xls_file} --outdir . >/dev/null
                              check_error
                              ;;
                           *) echo "You need to modify script '$0' to handle '$found_prog'."
                              abort -1
                              ;;
      esac

      [ ! -f "${csv_file}" ] && echo "Could not convert '${xls_file}' to '${csv_file}'" && abort -1

done

echo "All files converted successfully from ${filetype} to csv."

exit 0
